#pylint: skip-file
import unittest
from Atomic.backendutils import BackendUtils
from Atomic.backends._docker import DockerBackend
from Atomic.backends._ostree import OSTreeBackend
from Atomic.info import Info
from Atomic.images import Images
from Atomic.verify import Verify
from Atomic.objects.image import Image

no_mock = True
try:
    from unittest.mock import MagicMock, patch
    no_mock = False
except ImportError:
    try:
        from mock import MagicMock, patch
        no_mock = False
    except ImportError:
        # Mock is already set to False
        pass


_centos_inspect_image = {u'Comment': u'', u'Container': u'58aeaa4866c2845b48ab998b7cba3856a9fb64a681f92544cb035b85066b5102', u'DockerVersion': u'1.12.1', u'Parent': u'', u'Created': u'2016-11-02T19:52:09.463959047Z', u'Config': {u'Tty': False, u'Cmd': [u'/bin/bash'], u'Volumes': None, u'Domainname': u'', u'WorkingDir': u'', u'Image': u'5a2725191d75eb64e9b7c969cd23d8c67c6e8af9979e521a417bbfa34434fb83', u'Hostname': u'd6dcf178f680', u'StdinOnce': False, u'Labels': {u'build-date': u'20161102', u'vendor': u'CentOS', u'name': u'CentOS Base Image', u'license': u'GPLv2'}, u'AttachStdin': False, u'User': u'', u'Env': [u'PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin'], u'Entrypoint': None, u'OnBuild': None, u'AttachStderr': False, u'AttachStdout': False, u'OpenStdin': False}, u'Author': u'https://github.com/CentOS/sig-cloud-instance-images', u'GraphDriver': {u'Data': {u'DeviceName': u'docker-253:1-20984667-e3af0c61256f885331fb1a3adc27ea509a10ba9a0ba9175c1a149f81bddcd30d', u'DeviceSize': u'10737418240', u'DeviceId': u'2'}, u'Name': u'devicemapper'}, u'VirtualSize': 196509652, u'Os': u'linux', u'Architecture': u'amd64', u'ContainerConfig': {u'Tty': False, u'Cmd': [u'/bin/sh', u'-c', u'#(nop) ', u'CMD ["/bin/bash"]'], u'Volumes': None, u'Domainname': u'', u'WorkingDir': u'', u'Image': u'5a2725191d75eb64e9b7c969cd23d8c67c6e8af9979e521a417bbfa34434fb83', u'Hostname': u'd6dcf178f680', u'StdinOnce': False, u'Labels': {u'build-date': u'20161102', u'vendor': u'CentOS', u'name': u'CentOS Base Image', u'license': u'GPLv2'}, u'AttachStdin': False, u'User': u'', u'Env': [u'PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin'], u'Entrypoint': None, u'OnBuild': None, u'AttachStderr': False, u'AttachStdout': False, u'OpenStdin': False}, u'Size': 196509652, u'RepoDigests': [u'docker.io/centos@b2f9d1c0ff5f87a4743104d099a3d561002ac500db1b9bfa02a783a46e0d366c'], u'Id': u'0584b3d2cf6d235ee310cf14b54667d889887b838d3f3d3033acd70fc3c48b8a', u'RepoTags': [u'docker.io/centos:latest']}
_docker_centos_result = 'Image Name: docker.io/library/centos:latest\nbuild-date: 20161102\nlicense: GPLv2\nname: CentOS Base Image\nvendor: CentOS\n'
_ostree_centos_result = 'Image Name: docker.io/library/centos:latest\nbuild-date: 20161102\nlicense: GPLv2\nname: CentOS Base Image\nvendor: CentOS\n\n\nTemplate variables with default value, but overridable with --set:\nRUN_DIRECTORY: {SET_BY_OS}\nSTATE_DIRECTORY: {SET_BY_OS}\n'
_centos_ostree_inspect = {'Version': 'centos', 'Labels': {u'build-date': u'20161102', u'vendor': u'CentOS', u'name': u'CentOS Base Image', u'license': u'GPLv2'}, 'Names': [], 'Created': 1480352808, 'OSTree-rev': 'd2122127d30f94ae12ebe5afa542abdb1870201b0b9750bae3ceb74aa6ed18e6', 'RepoTags': ['centos'], 'Id': u'b2f9d1c0ff5f87a4743104d099a3d561002ac500db1b9bfa02a783a46e0d366c', 'ImageType': 'system', 'ImageId': u'b2f9d1c0ff5f87a4743104d099a3d561002ac500db1b9bfa02a783a46e0d366c'}
_rhel_docker_inspect = {u'Comment': u'', u'Container': u'', u'DockerVersion': u'1.9.1', u'Parent': u'', u'Created': u'2016-10-26T12:02:33.368772Z', u'Config': {u'Tty': False, u'Cmd': [u'/bin/bash'], u'Volumes': None, u'Domainname': u'', u'WorkingDir': u'', u'Image': u'f6f6121b053b2312688c87d3a1d32d06a984dc01d2ea7738508a50581cddb6b4', u'Hostname': u'', u'StdinOnce': False, u'Labels': {u'com.redhat.component': u'rhel-server-docker', u'authoritative-source-url': u'registry.access.redhat.com', u'distribution-scope': u'public', u'Vendor': u'Red Hat, Inc.', u'Name': u'rhel7/rhel', u'Build_Host': u'rcm-img01.build.eng.bos.redhat.com', u'vcs-type': u'git', u'name': u'rhel7/rhel', u'vcs-ref': u'7eeaf203cf909c2c056fba7066db9c1073a28d97', u'release': u'45', u'Version': u'7.3', u'Architecture': u'x86_64', u'version': u'7.3', u'Release': u'45', u'vendor': u'Red Hat, Inc.', u'BZComponent': u'rhel-server-docker', u'build-date': u'2016-10-26T07:54:17.037911Z', u'com.redhat.build-host': u'ip-10-29-120-48.ec2.internal', u'architecture': u'x86_64'}, u'AttachStdin': False, u'User': u'', u'Env': [u'PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin', u'container=docker'], u'Entrypoint': None, u'OnBuild': [], u'AttachStderr': False, u'AttachStdout': False, u'OpenStdin': False}, u'Author': u'Red Hat, Inc.', u'GraphDriver': {u'Data': {u'DeviceName': u'docker-253:2-5900125-a2bce97a4fd7ea12dce9865caa461ead8d1caf51ef452aba2f1b9d98efdf968f', u'DeviceSize': u'10737418240', u'DeviceId': u'623'}, u'Name': u'devicemapper'}, u'VirtualSize': 192508958, u'Os': u'linux', u'Architecture': u'amd64', u'RootFS': {u'Layers': [u'34d3e0e77091d9d51c6f70a7a7a4f7536aab214a55e02a8923af8f80cbe60d30', u'ccd6fc81ec49bd45f04db699401eb149b1945bb7292476b390ebdcdd7d975697'], u'Type': u'layers'}, u'ContainerConfig': {u'Tty': False, u'Cmd': None, u'Volumes': None, u'Domainname': u'', u'WorkingDir': u'', u'Image': u'', u'Hostname': u'', u'StdinOnce': False, u'Labels': None, u'AttachStdin': False, u'User': u'', u'Env': None, u'Entrypoint': None, u'OnBuild': None, u'AttachStderr': False, u'AttachStdout': False, u'OpenStdin': False}, u'Size': 192508958, u'RepoDigests': [u'registry.access.redhat.com/rhel7@sha256:da8a3e9297da7ccd1948366103d13c45b7e77489382351a777a7326004b63a21'], u'Id': u'f98706e16e41e56c4beaeea9fa77cd00fe35693635ed274f128876713afc0a1e', u'RepoTags': [u'registry.access.redhat.com/rhel7:latest']}
_rhel_version_result = 'IMAGE NAME                                       VERSION           IMAGE ID  \nregistry.access.redhat.com/rhel7:latest          rhel7/rhel-7.3-45 f98706e16e41\n'
_rhel_version_json = [{'Image': ['registry.access.redhat.com/rhel7:latest'], 'iid': 'f98706e16e41e56c4beaeea9fa77cd00fe35693635ed274f128876713afc0a1e', 'Version': '7.3-45'}]
_rhel_ostree_inspect = {'Created': 1480459483, 'RepoTags': ['registry.access.redhat.com/rhel7'], 'ImageId': 'da8a3e9297da7ccd1948366103d13c45b7e77489382351a777a7326004b63a21', 'OSTree-rev': 'a5840a53b233fdb936de8bcd35c77007b9af2d446ebabce6a2716772d6f089bd', 'Names': [], 'Id': 'da8a3e9297da7ccd1948366103d13c45b7e77489382351a777a7326004b63a21', 'Labels': {'name': 'rhel7/rhel', 'vendor': 'Red Hat, Inc.', 'authoritative-source-url': 'registry.access.redhat.com', 'version': '7.3', 'com.redhat.component': 'rhel-server-docker', 'distribution-scope': 'public', 'Name': 'rhel7/rhel', 'Architecture': 'x86_64', 'Version': '7.3', 'Vendor': 'Red Hat, Inc.', 'build-date': '2016-10-26T07:54:17.037911Z', 'com.redhat.build-host': 'ip-10-29-120-48.ec2.internal', 'Release': '45', 'BZComponent': 'rhel-server-docker', 'release': '45', 'vcs-type': 'git', 'Build_Host': 'rcm-img01.build.eng.bos.redhat.com', 'vcs-ref': '7eeaf203cf909c2c056fba7066db9c1073a28d97', 'architecture': 'x86_64'}, 'Version': 'registry.access.redhat.com/rhel7', 'ImageType': 'system'}
_rhel_ostree_version_result = 'IMAGE NAME                                VERSION                                 IMAGE ID  \nregistry.access.redhat.com/rhel7          registry.access.redhat.com/rhel7-7.3-45 da8a3e9297da\n'
_rhel_ostree_json = [{'Image': ['registry.access.redhat.com/rhel7'], 'iid': 'da8a3e9297da7ccd1948366103d13c45b7e77489382351a777a7326004b63a21', 'Version': '7.3-45'}]



@unittest.skipIf(no_mock, "Mock not found")
class TestInfo(unittest.TestCase):
    class Args():
        def __init__(self):
            self.storage = None
            self.force = False
            self.json = False
            self.debug = False

    def test_docker_info(self):
        db = DockerBackend()
        db._inspect_image = MagicMock(return_value=_centos_inspect_image)
        img_obj = db.inspect_image('docker.io/library/centos:latest')
        info = Info()
        args = self.Args()
        args.storage = 'docker'
        info.set_args(args)
        info.beu.get_backend_and_image_obj = MagicMock(return_value=(db, img_obj))
        result = info.info()
        self.assertEqual(result, _docker_centos_result)

    def test_ostree_info(self):
        ob = OSTreeBackend()
        ob.syscontainers.inspect_system_image = MagicMock(return_value=_centos_ostree_inspect)
        img_obj = ob.inspect_image('docker.io/library/centos:latest')
        img_obj._template_variables_set = {'RUN_DIRECTORY': '{SET_BY_OS}', 'STATE_DIRECTORY': '{SET_BY_OS}'}
        img_obj._template_variables_unset = {}
        info = Info()
        args = self.Args()
        args.storage = 'ostree'
        info.set_args(args)
        info.beu.get_backend_and_image_obj = MagicMock(return_value=(ob, img_obj))
        result = info.info()
        self.assertEqual(result, _ostree_centos_result)

    def test_version_json_docker(self):
        db = DockerBackend()
        db._inspect_image = MagicMock(return_value=_rhel_docker_inspect)
        img_obj = db.inspect_image('registry.access.redhat.com/rhel7')
        info = Info()
        info.get_layer_objects = MagicMock(return_value=img_obj.layers)
        args = self.Args()
        args.storage = 'docker'
        args.images = 'registry.access.redhat.com/rhel7'
        info.set_args(args)
        results = info.dbus_version()
        self.assertEqual(results, _rhel_version_json)

    def test_version_json_ostree(self):
        ob = OSTreeBackend()
        ob.syscontainers.inspect_system_image = MagicMock(return_value=_rhel_ostree_inspect)
        img_obj = ob.inspect_image('registry.access.redhat.com/rhel7')
        info = Info()
        info.get_layer_objects = MagicMock(return_value=img_obj.layers)
        args = self.Args()
        args.storage = 'ostree'
        args.images = 'registry.access.redhat.com/rhel7'
        info.set_args(args)
        results = info.dbus_version()
        self.assertEqual(results, _rhel_ostree_json)


_docker_images = [{'VirtualSize': 196509652, 'Labels': {'vendor': 'CentOS', 'license': 'GPLv2', 'build-date': '20161102', 'name': 'CentOS Base Image'}, 'RepoTags': ['docker.io/centos:latest'], 'ParentId': '', 'Id': '0584b3d2cf6d235ee310cf14b54667d889887b838d3f3d3033acd70fc3c48b8a', 'Size': 196509652, 'Created': 1478116329, 'RepoDigests': ['docker.io/centos@sha256:b2f9d1c0ff5f87a4743104d099a3d561002ac500db1b9bfa02a783a46e0d366c']}, {'VirtualSize': 1093484, 'Labels': {}, 'RepoTags': ['docker.io/busybox:latest'], 'ParentId': '', 'Id': 'e02e811dd08fd49e7f6032625495118e63f597eb150403d02e3238af1df240ba', 'Size': 1093484, 'Created': 1475874238, 'RepoDigests': ['docker.io/busybox@sha256:29f5d56d12684887bdfa50dcd29fc31eea4aaf4ad3bec43daf19026a7ce69912']}]
_system_images = [{'Labels': {'vendor': 'CentOS', 'license': 'GPLv2', 'build-date': '20161102', 'name': 'CentOS Base Image'}, 'ImageId': 'b2f9d1c0ff5f87a4743104d099a3d561002ac500db1b9bfa02a783a46e0d366c', 'Version': 'centos:latest', 'OSTree-rev': 'd2122127d30f94ae12ebe5afa542abdb1870201b0b9750bae3ceb74aa6ed18e6', 'ImageType': 'system', 'Id': 'b2f9d1c0ff5f87a4743104d099a3d561002ac500db1b9bfa02a783a46e0d366c', 'Created': 1480352808, 'Names': [], 'RepoTags': ['centos:latest']}, {'Labels': {}, 'ImageId': '29f5d56d12684887bdfa50dcd29fc31eea4aaf4ad3bec43daf19026a7ce69912', 'Version': 'busybox:latest', 'OSTree-rev': 'f0cbd09116e348782fc353f99db2b111a59fdf929e9a0180f3a8450c145ed8bc', 'ImageType': 'system', 'Id': '29f5d56d12684887bdfa50dcd29fc31eea4aaf4ad3bec43daf19026a7ce69912', 'Created': 1480348080, 'Names': [], 'RepoTags': ['busybox:latest']}]

@unittest.skipIf(no_mock, "Mock not found")
class TestImages(unittest.TestCase):
    class Args():
        def __init__(self):
            self.storage = None
            self.force = False
            self.json = False
            self.debug = False
            self.name = None
            self.image = None
            self.all = False

    def test_images(self):
        db = DockerBackend()
        db._inspect_image = MagicMock(return_value=_docker_images)
        ob = OSTreeBackend()
        ob.syscontainers.get_system_images = MagicMock(return_value=_system_images)
        images = Images()
        args = self.Args()
        args.storage = 'docker'
        args.json = True
        args.filter = None
        images.set_args(args)
        return_value = images.display_all_image_info()
        self.assertEqual(return_value, 0)


remote_centos_inspect_latest = {'Layers': [u'sha256:08d48e6f1cff259389732d35307bb877215fa28867cdaff50c1dbd6e0b993c1f'], 'Name': 'docker.io/library/centos', 'Created': u'2016-11-02T19:52:09.463959047Z', 'Architecture': u'amd64', 'Os': u'linux', 'id': u'sha256:0584b3d2cf6d235ee310cf14b54667d889887b838d3f3d3033acd70fc3c48b8a', 'RepoTags': [u'5.11', u'5', u'6.6', u'6.7', u'6.8', u'6', u'7.0.1406', u'7.1.1503', u'7.2.1511', u'7', u'centos5.11', u'centos5', u'centos6.6', u'centos6.7', u'centos6.8', u'centos6', u'centos7.0.1406', u'centos7.1.1503', u'centos7.2.1511', u'centos7', u'latest'], 'DockerVersion': u'1.12.1', 'Labels': {u'build-date': u'20161102', u'vendor': u'CentOS', u'name': u'CentOS Base Image', u'license': u'GPLv2'}, 'Tag': 'latest', 'Digest': 'sha256:b2f9d1c0ff5f87a4743104d099a3d561002ac500db1b9bfa02a783a46e0d366c'}
local_centos_inspect_old = {'Id': '16e9fdecc1febc87fb1ca09271009cf5f28eb8d4aec5515922ef298c145a6726', 'RepoDigests': ['docker.io/centos@sha256:7793b39617b28c6cd35774c00383b89a1265f3abf6efcaf0b8f4aafe4e0662d2'], 'Parent': '', 'GraphDriver': {'Name': 'devicemapper', 'Data': {'DeviceSize': '10737418240', 'DeviceName': 'docker-253:2-5900125-3fb5b406e6a53142129237c9e2c3a1ce8b6cf269b5f8071fcd62107c41544cd2', 'DeviceId': '779'}}, 'Created': '2016-08-30T18:20:19.39890162Z', 'Comment': '', 'DockerVersion': '1.12.1', 'VirtualSize': 210208812, 'Author': 'The CentOS Project <cloud-ops@centos.org> - ami_creator', 'Os': 'linux', 'RootFS': {'Type': 'layers', 'Layers': ['5fa0fa02637842ab1ddc8b3a17b86691c87c87d20800e6a95a113343f6ffd84c']}, 'Container': 'a5b0819aa82c224095e1a18e9df0776a7b38d32bacca073f054723b65fb54f0e', 'Architecture': 'amd64', 'RepoTags': ['docker.io/centos:centos7.0.1406'], 'Config': {'Labels': {}, 'Entrypoint': None, 'StdinOnce': False, 'OnBuild': None, 'Env': ['PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin'], 'Volumes': None, 'Cmd': None, 'User': '', 'AttachStdin': False, 'AttachStderr': False, 'AttachStdout': False, 'WorkingDir': '', 'Tty': False, 'Image': '20ae10d641a0af6f25ceaa75fdcf591d171e3c521a54a3f3a2868b602d735e11', 'Hostname': 'a5b0819aa82c', 'Domainname': '', 'OpenStdin': False}, 'Size': 210208812, 'ContainerConfig': {'Labels': {}, 'Entrypoint': None, 'StdinOnce': False, 'OnBuild': None, 'Env': ['PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin'], 'Volumes': None, 'Cmd': ['/bin/sh', '-c', '#(nop) ADD file:6a409eac27f0c7e04393da096dbeff01b929405e79b15222a0dc06a2084d3df3 in / '], 'User': '', 'AttachStdin': False, 'AttachStderr': False, 'AttachStdout': False, 'WorkingDir': '', 'Tty': False, 'Image': '20ae10d641a0af6f25ceaa75fdcf591d171e3c521a54a3f3a2868b602d735e11', 'Hostname': 'a5b0819aa82c', 'Domainname': '', 'OpenStdin': False}}
docker_dbus_result = [{'differs': False, 'local_version': '0584b3d2cf6d235ee310cf14b54667d889887b838d3f3d3033acd70fc3c48b8a', 'name': 'CentOS Base Image', 'remote_version': '0584b3d2cf6d235ee310cf14b54667d889887b838d3f3d3033acd70fc3c48b8a'}]


@unittest.skipIf(no_mock, "Mock not found")
class TestVerify(unittest.TestCase):
    class Args():
        def __init__(self):
            self.storage = None
            self.debug = False
            self.name = None
            self.image = None

    def test_verify_docker_same(self):
        with patch('Atomic.backendutils.BackendUtils.get_backend_and_image_obj') as mockobj:
            args = self.Args()
            args.storage = 'docker'
            args.image = 'docker.io/library/centos:latest'
            db = DockerBackend()
            db._inspect_image = MagicMock(return_value=_centos_inspect_image)
            local_image_object = db.inspect_image(args.image)
            mockobj.return_value = (db, local_image_object)
            v = Verify()
            v.set_args(args)
            Image.remote_inspect = MagicMock(return_value=remote_centos_inspect_latest)
            self.assertEqual(v.verify_dbus(), docker_dbus_result)

    def test_verify_docker_diff(self):
        with patch('Atomic.backendutils.BackendUtils.get_backend_and_image_obj') as mockobj:
            args = self.Args()
            args.storage = 'docker'
            args.image = 'docker.io/library/centos:centos7.0.1406'
            db = DockerBackend()
            db._inspect_image = MagicMock(return_value=local_centos_inspect_old)
            local_image_object = db.inspect_image(args.image)
            mockobj.return_value = (db, local_image_object)
            v = Verify()
            v.set_args(args)
            Image.remote_inspect = MagicMock(return_value=remote_centos_inspect_latest)
            self.assertNotEqual(v.verify_dbus(), docker_dbus_result)

    def test_verify_ostree_same(self):
        # I hit some issue with syscontainers here.  Will leave until that issue
        # can be resolved with guiseppe
        pass



#pylint: skip-file

import unittest
from Atomic.backendutils import BackendUtils
from Atomic.backends._docker import DockerBackend
from Atomic.backends._ostree import OSTreeBackend

no_mock = True
try:
    from unittest.mock import MagicMock, patch
    no_mock = False
except ImportError:
    try:
        from mock import MagicMock, patch
        no_mock = False
    except ImportError:
        # Mock is already set to False
        pass


BACKENDS = [DockerBackend, OSTreeBackend]
d_images = [{u'Created': 1478116329, u'Labels': {u'build-date': u'20161102', u'vendor': u'CentOS', u'name': u'CentOS Base Image', u'license': u'GPLv2'}, u'VirtualSize': 196509652, u'ParentId': u'', u'RepoTags': [u'docker.io/centos:latest'], u'RepoDigests': [u'docker.io/centos@b2f9d1c0ff5f87a4743104d099a3d561002ac500db1b9bfa02a783a46e0d366c'], u'Id': u'0584b3d2cf6d235ee310cf14b54667d889887b838d3f3d3033acd70fc3c48b8a', u'Size': 196509652}, {u'Created': 1477910440, u'Labels': {u'Version': u'7.3', u'INSTALL': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/install.sh', u'vendor': u'Red Hat, Inc.', u'vcs-ref': u'fabbc9b2d819e919444fced9b24db3953ab254e6', u'authoritative-source-url': u'registry.access.redhat.com', u'Vendor': u'Red Hat, Inc.', u'version': u'7.3', u'com.redhat.component': u'rsyslog-docker', u'distribution-scope': u'public', u'run': u'docker run -d --privileged --name NAME --net=host --pid=host -v /etc/pki/rsyslog:/etc/pki/rsyslog -v /etc/rsyslog.conf:/etc/rsyslog.conf -v /etc/sysconfig/rsyslog:/etc/sysconfig/rsyslog -v /etc/rsyslog.d:/etc/rsyslog.d -v /var/log:/var/log -v /var/lib/rsyslog:/var/lib/rsyslog -v /run:/run -v /etc/machine-id:/etc/machine-id -v /etc/localtime:/etc/localtime -e IMAGE=IMAGE -e NAME=NAME --restart=always IMAGE /bin/rsyslog.sh', u'Name': u'rhel7/rsyslog', u'License': u'GPLv3', u'Build_Host': u'ip-10-29-120-226.ec2.internal', u'vcs-type': u'git', u'Architecture': u'x86_64', u'Release': u'8', u'BZComponent': u'rsyslog-docker', u'build-date': u'2016-10-31T06:37:07.885264Z', u'com.redhat.build-host': u'ip-10-29-120-226.ec2.internal', u'UNINSTALL': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/uninstall.sh', u'RUN': u'docker run -d --privileged --name NAME --net=host --pid=host -v /etc/pki/rsyslog:/etc/pki/rsyslog -v /etc/rsyslog.conf:/etc/rsyslog.conf -v /etc/sysconfig/rsyslog:/etc/sysconfig/rsyslog -v /etc/rsyslog.d:/etc/rsyslog.d -v /var/log:/var/log -v /var/lib/rsyslog:/var/lib/rsyslog -v /run:/run -v /etc/machine-id:/etc/machine-id -v /etc/localtime:/etc/localtime -e IMAGE=IMAGE -e NAME=NAME --restart=always IMAGE /bin/rsyslog.sh', u'name': u'rhel7/rsyslog', u'architecture': u'x86_64', u'install': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/install.sh', u'release': u'8', u'uninstall': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/uninstall.sh'}, u'VirtualSize': 205238327, u'ParentId': u'', u'RepoTags': [u'registry.access.redhat.com/rhel7/rsyslog:latest'], u'RepoDigests': [u'registry.access.redhat.com/rhel7/rsyslog@729f781838054ce6085ab96a8fed45b7fdb2ed3dab2f2e1fb1b081b32e74d86c'], u'Id': u'04f7e9543b939537000296e38c11d5d422653f74a6dd2a3905b931bb8f4c3265', u'Size': 205238327}, {u'Created': 1477483353, u'Labels': {u'com.redhat.component': u'rhel-server-docker', u'authoritative-source-url': u'registry.access.redhat.com', u'distribution-scope': u'public', u'Vendor': u'Red Hat, Inc.', u'Name': u'rhel7/rhel', u'Build_Host': u'rcm-img01.build.eng.bos.redhat.com', u'vcs-type': u'git', u'name': u'rhel7/rhel', u'vcs-ref': u'7eeaf203cf909c2c056fba7066db9c1073a28d97', u'release': u'45', u'Version': u'7.3', u'Architecture': u'x86_64', u'version': u'7.3', u'Release': u'45', u'vendor': u'Red Hat, Inc.', u'BZComponent': u'rhel-server-docker', u'build-date': u'2016-10-26T07:54:17.037911Z', u'com.redhat.build-host': u'ip-10-29-120-48.ec2.internal', u'architecture': u'x86_64'}, u'VirtualSize': 192508958, u'ParentId': u'', u'RepoTags': [u'registry.access.redhat.com/rhel7:latest'], u'RepoDigests': [u'registry.access.redhat.com/rhel7@da8a3e9297da7ccd1948366103d13c45b7e77489382351a777a7326004b63a21'], u'Id': u'f98706e16e41e56c4beaeea9fa77cd00fe35693635ed274f128876713afc0a1e', u'Size': 192508958}, {u'Created': 1475874238, u'Labels': {}, u'VirtualSize': 1093484, u'ParentId': u'', u'RepoTags': [u'docker.io/busybox:latest'], u'RepoDigests': [u'docker.io/busybox@29f5d56d12684887bdfa50dcd29fc31eea4aaf4ad3bec43daf19026a7ce69912'], u'Id': u'e02e811dd08fd49e7f6032625495118e63f597eb150403d02e3238af1df240ba', u'Size': 1093484}, {u'Created': 1456801947, u'Labels': {u'Vendor': u'Red Hat, Inc.', u'Name': u'rhel7/rhel', u'Build_Host': u'rcm-img02.build.eng.bos.redhat.com', u'Version': u'7.2', u'Architecture': u'x86_64', u'Release': u'46', u'BZComponent': u'rhel-server-docker', u'Authoritative_Registry': u'registry.access.redhat.com'}, u'VirtualSize': 203247534, u'ParentId': u'', u'RepoTags': [u'registry.access.redhat.com/rhel7:7.2-46'], u'RepoDigests': [u'registry.access.redhat.com/rhel7@6b76f95ab3ed949c2ca3110765f310bf8f4c9333394efba11727b14863c73f10'], u'Id': u'32f8a1d5f01956ca38f3887a83b1736da4f450d6132563c665bdc0a7a4684c4d', u'Size': 203247534}, {u'Created': 1455812978, u'Labels': {u'Vendor': u'Red Hat, Inc.', u'Name': u'rhel7/rhel', u'Build_Host': u'rcm-img01.build.eng.bos.redhat.com', u'Version': u'7.2', u'Architecture': u'x86_64', u'Release': u'44', u'BZComponent': u'rhel-server-docker', u'Authoritative_Registry': u'registry.access.redhat.com'}, u'VirtualSize': 203243624, u'ParentId': u'', u'RepoTags': [u'registry.access.redhat.com/rhel7:7.2-44'], u'RepoDigests': [u'registry.access.redhat.com/rhel7@af41b2f4e4fd2260247f1450b27da021d319ca4413e57ae833313398334d6e6f'], u'Id': u'da3ab9be742254035a2736da5d41c0a941b9a12ddd29f70b6129748e3749dd01', u'Size': 203243624}]
d_containers = [{u'Status': u'Exited (0) 51 seconds ago', u'Created': 1478633817, u'Image': u'centos', u'Labels': {u'build-date': u'20161102', u'vendor': u'CentOS', u'name': u'CentOS Base Image', u'license': u'GPLv2'}, u'NetworkSettings': {u'Networks': {u'bridge': {u'NetworkID': u'', u'MacAddress': u'', u'GlobalIPv6PrefixLen': 0, u'Links': None, u'GlobalIPv6Address': u'', u'IPv6Gateway': u'', u'IPAMConfig': None, u'EndpointID': u'', u'IPPrefixLen': 0, u'IPAddress': u'', u'Gateway': u'', u'Aliases': None}}}, u'HostConfig': {u'NetworkMode': u'default'}, u'ImageID': u'0584b3d2cf6d235ee310cf14b54667d889887b838d3f3d3033acd70fc3c48b8a', u'Command': u'/bin/bash', u'Names': [u'/happy_darwin'], u'Id': u'7002c7a08bfd3427f7eee23a5f54981a8637d1b3b3c3787ca94255273a1d7e43', u'Ports': []}, {u'Status': u'Up About a minute', u'Created': 1478633803, u'Image': u'registry.access.redhat.com/rhel7/rsyslog', u'Labels': {u'Version': u'7.3', u'INSTALL': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/install.sh', u'vendor': u'Red Hat, Inc.', u'vcs-ref': u'fabbc9b2d819e919444fced9b24db3953ab254e6', u'authoritative-source-url': u'registry.access.redhat.com', u'Vendor': u'Red Hat, Inc.', u'version': u'7.3', u'com.redhat.component': u'rsyslog-docker', u'distribution-scope': u'public', u'run': u'docker run -d --privileged --name NAME --net=host --pid=host -v /etc/pki/rsyslog:/etc/pki/rsyslog -v /etc/rsyslog.conf:/etc/rsyslog.conf -v /etc/sysconfig/rsyslog:/etc/sysconfig/rsyslog -v /etc/rsyslog.d:/etc/rsyslog.d -v /var/log:/var/log -v /var/lib/rsyslog:/var/lib/rsyslog -v /run:/run -v /etc/machine-id:/etc/machine-id -v /etc/localtime:/etc/localtime -e IMAGE=IMAGE -e NAME=NAME --restart=always IMAGE /bin/rsyslog.sh', u'Name': u'rhel7/rsyslog', u'License': u'GPLv3', u'Build_Host': u'ip-10-29-120-226.ec2.internal', u'vcs-type': u'git', u'Architecture': u'x86_64', u'Release': u'8', u'BZComponent': u'rsyslog-docker', u'build-date': u'2016-10-31T06:37:07.885264Z', u'com.redhat.build-host': u'ip-10-29-120-226.ec2.internal', u'UNINSTALL': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/uninstall.sh', u'RUN': u'docker run -d --privileged --name NAME --net=host --pid=host -v /etc/pki/rsyslog:/etc/pki/rsyslog -v /etc/rsyslog.conf:/etc/rsyslog.conf -v /etc/sysconfig/rsyslog:/etc/sysconfig/rsyslog -v /etc/rsyslog.d:/etc/rsyslog.d -v /var/log:/var/log -v /var/lib/rsyslog:/var/lib/rsyslog -v /run:/run -v /etc/machine-id:/etc/machine-id -v /etc/localtime:/etc/localtime -e IMAGE=IMAGE -e NAME=NAME --restart=always IMAGE /bin/rsyslog.sh', u'name': u'rhel7/rsyslog', u'architecture': u'x86_64', u'install': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/install.sh', u'release': u'8', u'uninstall': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/uninstall.sh'}, u'NetworkSettings': {u'Networks': {u'host': {u'NetworkID': u'', u'MacAddress': u'', u'GlobalIPv6PrefixLen': 0, u'Links': None, u'GlobalIPv6Address': u'', u'IPv6Gateway': u'', u'IPAMConfig': None, u'EndpointID': u'e4c76518434d598f8bcfd86fa3c377cb7d1ff4fdcad17184e42ef51069aa7125', u'IPPrefixLen': 0, u'IPAddress': u'', u'Gateway': u'', u'Aliases': None}}}, u'HostConfig': {u'NetworkMode': u'host'}, u'ImageID': u'04f7e9543b939537000296e38c11d5d422653f74a6dd2a3905b931bb8f4c3265', u'Command': u'/bin/rsyslog.sh', u'Names': [u'/rsyslog'], u'Id': u'3e014f35a7d0a22579cdad033edefd287fe01cac0e52dd68162e477fc150dc64', u'Ports': []}]
_rsyslog_container_inspect = {u'ExecIDs': None, u'State': {u'Status': u'running', u'Pid': 3425, u'OOMKilled': False, u'Dead': False, u'Paused': False, u'Running': True, u'FinishedAt': u'0001-01-01T00:00:00Z', u'Restarting': False, u'Error': u'', u'StartedAt': u'2016-11-08T19:36:44.44460996Z', u'ExitCode': 0}, u'Config': {u'Tty': False, u'Cmd': [u'/bin/rsyslog.sh'], u'Volumes': None, u'Domainname': u'localdomain', u'WorkingDir': u'', u'Image': u'registry.access.redhat.com/rhel7/rsyslog', u'Hostname': u'atomic-baude', u'StdinOnce': False, u'Labels': {u'Version': u'7.3', u'INSTALL': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/install.sh', u'vendor': u'Red Hat, Inc.', u'vcs-ref': u'fabbc9b2d819e919444fced9b24db3953ab254e6', u'authoritative-source-url': u'registry.access.redhat.com', u'Vendor': u'Red Hat, Inc.', u'version': u'7.3', u'com.redhat.component': u'rsyslog-docker', u'distribution-scope': u'public', u'run': u'docker run -d --privileged --name NAME --net=host --pid=host -v /etc/pki/rsyslog:/etc/pki/rsyslog -v /etc/rsyslog.conf:/etc/rsyslog.conf -v /etc/sysconfig/rsyslog:/etc/sysconfig/rsyslog -v /etc/rsyslog.d:/etc/rsyslog.d -v /var/log:/var/log -v /var/lib/rsyslog:/var/lib/rsyslog -v /run:/run -v /etc/machine-id:/etc/machine-id -v /etc/localtime:/etc/localtime -e IMAGE=IMAGE -e NAME=NAME --restart=always IMAGE /bin/rsyslog.sh', u'Name': u'rhel7/rsyslog', u'License': u'GPLv3', u'Build_Host': u'ip-10-29-120-226.ec2.internal', u'vcs-type': u'git', u'Architecture': u'x86_64', u'Release': u'8', u'BZComponent': u'rsyslog-docker', u'build-date': u'2016-10-31T06:37:07.885264Z', u'com.redhat.build-host': u'ip-10-29-120-226.ec2.internal', u'UNINSTALL': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/uninstall.sh', u'RUN': u'docker run -d --privileged --name NAME --net=host --pid=host -v /etc/pki/rsyslog:/etc/pki/rsyslog -v /etc/rsyslog.conf:/etc/rsyslog.conf -v /etc/sysconfig/rsyslog:/etc/sysconfig/rsyslog -v /etc/rsyslog.d:/etc/rsyslog.d -v /var/log:/var/log -v /var/lib/rsyslog:/var/lib/rsyslog -v /run:/run -v /etc/machine-id:/etc/machine-id -v /etc/localtime:/etc/localtime -e IMAGE=IMAGE -e NAME=NAME --restart=always IMAGE /bin/rsyslog.sh', u'name': u'rhel7/rsyslog', u'architecture': u'x86_64', u'install': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/install.sh', u'release': u'8', u'uninstall': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/uninstall.sh'}, u'AttachStdin': False, u'User': u'', u'Env': [u'IMAGE=registry.access.redhat.com/rhel7/rsyslog', u'NAME=rsyslog', u'PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin', u'container=docker'], u'Entrypoint': None, u'OnBuild': None, u'AttachStderr': False, u'AttachStdout': False, u'OpenStdin': False}, u'ResolvConfPath': u'/var/lib/docker/containers/3e014f35a7d0a22579cdad033edefd287fe01cac0e52dd68162e477fc150dc64/resolv.conf', u'HostsPath': u'/var/lib/docker/containers/3e014f35a7d0a22579cdad033edefd287fe01cac0e52dd68162e477fc150dc64/hosts', u'Args': [], u'Driver': u'devicemapper', u'Path': u'/bin/rsyslog.sh', u'HostnamePath': u'/var/lib/docker/containers/3e014f35a7d0a22579cdad033edefd287fe01cac0e52dd68162e477fc150dc64/hostname', u'RestartCount': 0, u'Name': u'/rsyslog', u'Created': u'2016-11-08T19:36:43.024272115Z', u'GraphDriver': {u'Data': {u'DeviceName': u'docker-253:1-20984667-ca3eae26c63b8bd867536b53b7e5fade1eae923a2999864b31546f06eb19b123', u'DeviceSize': u'10737418240', u'DeviceId': u'10'}, u'Name': u'devicemapper'}, u'Mounts': [{u'RW': True, u'Source': u'/etc/pki/rsyslog', u'Destination': u'/etc/pki/rsyslog', u'Mode': u'', u'Propagation': u'rslave'}, {u'RW': True, u'Source': u'/var/lib/rsyslog', u'Destination': u'/var/lib/rsyslog', u'Mode': u'', u'Propagation': u'rslave'}, {u'RW': True, u'Source': u'/run', u'Destination': u'/run', u'Mode': u'', u'Propagation': u'rslave'}, {u'RW': True, u'Source': u'/etc/sysconfig/rsyslog', u'Destination': u'/etc/sysconfig/rsyslog', u'Mode': u'', u'Propagation': u'rslave'}, {u'RW': True, u'Source': u'/var/log', u'Destination': u'/var/log', u'Mode': u'', u'Propagation': u'rslave'}, {u'RW': True, u'Source': u'/etc/rsyslog.conf', u'Destination': u'/etc/rsyslog.conf', u'Mode': u'', u'Propagation': u'rslave'}, {u'RW': True, u'Source': u'/etc/machine-id', u'Destination': u'/etc/machine-id', u'Mode': u'', u'Propagation': u'rslave'}, {u'RW': True, u'Source': u'/etc/localtime', u'Destination': u'/etc/localtime', u'Mode': u'', u'Propagation': u'rslave'}, {u'RW': True, u'Source': u'/etc/rsyslog.d', u'Destination': u'/etc/rsyslog.d', u'Mode': u'', u'Propagation': u'rslave'}], u'ProcessLabel': u'', u'NetworkSettings': {u'Bridge': u'', u'Networks': {u'host': {u'NetworkID': u'f5f370f21b319d55187d4c05eabb4dfce1d866b8d5ad02069c16c9c59067bc2e', u'MacAddress': u'', u'GlobalIPv6PrefixLen': 0, u'Links': None, u'GlobalIPv6Address': u'', u'IPv6Gateway': u'', u'IPAMConfig': None, u'EndpointID': u'e4c76518434d598f8bcfd86fa3c377cb7d1ff4fdcad17184e42ef51069aa7125', u'IPPrefixLen': 0, u'IPAddress': u'', u'Gateway': u'', u'Aliases': None}}, u'SecondaryIPv6Addresses': None, u'LinkLocalIPv6Address': u'', u'HairpinMode': False, u'IPv6Gateway': u'', u'SecondaryIPAddresses': None, u'SandboxID': u'eb35aa677a88a07be28fd08c0bfae2d4eafc3a56637cc07f34acd31d8ac740de', u'MacAddress': u'', u'GlobalIPv6Address': u'', u'Gateway': u'', u'LinkLocalIPv6PrefixLen': 0, u'EndpointID': u'', u'SandboxKey': u'/var/run/docker/netns/default', u'GlobalIPv6PrefixLen': 0, u'IPPrefixLen': 0, u'IPAddress': u'', u'Ports': {}}, u'AppArmorProfile': u'', u'Image': u'04f7e9543b939537000296e38c11d5d422653f74a6dd2a3905b931bb8f4c3265', u'LogPath': u'', u'HostConfig': {u'CpuPeriod': 0, u'MemorySwappiness': -1, u'ContainerIDFile': u'', u'MemorySwap': 0, u'BlkioDeviceReadIOps': None, u'CpuQuota': 0, u'Dns': [], u'ExtraHosts': None, u'PidsLimit': 0, u'DnsSearch': [], u'Privileged': True, u'Ulimits': None, u'CpusetCpus': u'', u'CgroupParent': u'', u'BlkioWeight': 0, u'RestartPolicy': {u'MaximumRetryCount': 0, u'Name': u'always'}, u'OomScoreAdj': 0, u'BlkioDeviceReadBps': None, u'VolumeDriver': u'', u'ReadonlyRootfs': False, u'CpuShares': 0, u'PublishAllPorts': False, u'MemoryReservation': 0, u'BlkioWeightDevice': None, u'ConsoleSize': [0, 0], u'NetworkMode': u'host', u'BlkioDeviceWriteBps': None, u'Isolation': u'', u'GroupAdd': None, u'Devices': [], u'BlkioDeviceWriteIOps': None, u'Binds': [u'/etc/machine-id:/etc/machine-id', u'/etc/localtime:/etc/localtime', u'/var/lib/rsyslog:/var/lib/rsyslog', u'/run:/run', u'/etc/sysconfig/rsyslog:/etc/sysconfig/rsyslog', u'/etc/rsyslog.d:/etc/rsyslog.d', u'/var/log:/var/log', u'/etc/pki/rsyslog:/etc/pki/rsyslog', u'/etc/rsyslog.conf:/etc/rsyslog.conf'], u'CpusetMems': u'', u'KernelMemory': 0, u'UTSMode': u'', u'PidMode': u'host', u'VolumesFrom': None, u'CapDrop': None, u'DnsOptions': [], u'ShmSize': 67108864, u'Links': None, u'IpcMode': u'', u'PortBindings': {}, u'SecurityOpt': [u'label:disable'], u'CapAdd': None, u'Memory': 0, u'OomKillDisable': False, u'LogConfig': {u'Config': {}, u'Type': u'journald'}}, u'Id': u'3e014f35a7d0a22579cdad033edefd287fe01cac0e52dd68162e477fc150dc64', u'MountLabel': u''}
_rsyslog_image_inspect = {u'Comment': u'', u'Container': u'', u'DockerVersion': u'1.9.1', u'Parent': u'', u'Created': u'2016-10-31T10:40:40.771883Z', u'Config': {u'Tty': False, u'Cmd': [u'/bin/rsyslog.sh'], u'Volumes': None, u'Domainname': u'', u'WorkingDir': u'', u'Image': u'91c58d6c27c4a47876f1851b70cbcd416e3f84466730385f7d663e87b1d63a4f', u'Hostname': u'', u'StdinOnce': False, u'Labels': {u'Version': u'7.3', u'INSTALL': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/install.sh', u'vendor': u'Red Hat, Inc.', u'vcs-ref': u'fabbc9b2d819e919444fced9b24db3953ab254e6', u'authoritative-source-url': u'registry.access.redhat.com', u'Vendor': u'Red Hat, Inc.', u'version': u'7.3', u'com.redhat.component': u'rsyslog-docker', u'distribution-scope': u'public', u'run': u'docker run -d --privileged --name NAME --net=host --pid=host -v /etc/pki/rsyslog:/etc/pki/rsyslog -v /etc/rsyslog.conf:/etc/rsyslog.conf -v /etc/sysconfig/rsyslog:/etc/sysconfig/rsyslog -v /etc/rsyslog.d:/etc/rsyslog.d -v /var/log:/var/log -v /var/lib/rsyslog:/var/lib/rsyslog -v /run:/run -v /etc/machine-id:/etc/machine-id -v /etc/localtime:/etc/localtime -e IMAGE=IMAGE -e NAME=NAME --restart=always IMAGE /bin/rsyslog.sh', u'Name': u'rhel7/rsyslog', u'License': u'GPLv3', u'Build_Host': u'ip-10-29-120-226.ec2.internal', u'vcs-type': u'git', u'Architecture': u'x86_64', u'Release': u'8', u'BZComponent': u'rsyslog-docker', u'build-date': u'2016-10-31T06:37:07.885264Z', u'com.redhat.build-host': u'ip-10-29-120-226.ec2.internal', u'UNINSTALL': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/uninstall.sh', u'RUN': u'docker run -d --privileged --name NAME --net=host --pid=host -v /etc/pki/rsyslog:/etc/pki/rsyslog -v /etc/rsyslog.conf:/etc/rsyslog.conf -v /etc/sysconfig/rsyslog:/etc/sysconfig/rsyslog -v /etc/rsyslog.d:/etc/rsyslog.d -v /var/log:/var/log -v /var/lib/rsyslog:/var/lib/rsyslog -v /run:/run -v /etc/machine-id:/etc/machine-id -v /etc/localtime:/etc/localtime -e IMAGE=IMAGE -e NAME=NAME --restart=always IMAGE /bin/rsyslog.sh', u'name': u'rhel7/rsyslog', u'architecture': u'x86_64', u'install': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/install.sh', u'release': u'8', u'uninstall': u'docker run --rm --privileged -v /:/host -e HOST=/host -e IMAGE=IMAGE -e NAME=NAME IMAGE /bin/uninstall.sh'}, u'AttachStdin': False, u'User': u'', u'Env': [u'PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin', u'container=docker'], u'Entrypoint': None, u'OnBuild': [], u'AttachStderr': False, u'AttachStdout': False, u'OpenStdin': False}, u'Author': u'Red Hat, Inc.', u'GraphDriver': {u'Data': {u'DeviceName': u'docker-253:1-20984667-37714020f437aa4430146d44e91f1c067ac18b42265acd265d20e7ad674474e6', u'DeviceSize': u'10737418240', u'DeviceId': u'5'}, u'Name': u'devicemapper'}, u'VirtualSize': 205238327, u'Os': u'linux', u'Architecture': u'amd64', u'ContainerConfig': {u'Tty': False, u'Cmd': None, u'Volumes': None, u'Domainname': u'', u'WorkingDir': u'', u'Image': u'', u'Hostname': u'', u'StdinOnce': False, u'Labels': None, u'AttachStdin': False, u'User': u'', u'Env': None, u'Entrypoint': None, u'OnBuild': None, u'AttachStderr': False, u'AttachStdout': False, u'OpenStdin': False}, u'Size': 205238327, u'RepoDigests': [u'registry.access.redhat.com/rhel7/rsyslog@729f781838054ce6085ab96a8fed45b7fdb2ed3dab2f2e1fb1b081b32e74d86c'], u'Id': u'04f7e9543b939537000296e38c11d5d422653f74a6dd2a3905b931bb8f4c3265', u'RepoTags': [u'registry.access.redhat.com/rhel7/rsyslog:latest']}
_centos_name_search = [{'Created': 1478116329, 'RepoDigests': ['docker.io/centos@b2f9d1c0ff5f87a4743104d099a3d561002ac500db1b9bfa02a783a46e0d366c'], 'Id': '0584b3d2cf6d235ee310cf14b54667d889887b838d3f3d3033acd70fc3c48b8a', 'Labels': {'build-date': '20161102', 'license': 'GPLv2', 'name': 'CentOS Base Image', 'vendor': 'CentOS'}, 'Size': 196509652, 'RepoTags': ['docker.io/centos:latest'], 'VirtualSize': 196509652, 'ParentId': ''}]
_centos_inspect_image = {u'Comment': u'', u'Container': u'58aeaa4866c2845b48ab998b7cba3856a9fb64a681f92544cb035b85066b5102', u'DockerVersion': u'1.12.1', u'Parent': u'', u'Created': u'2016-11-02T19:52:09.463959047Z', u'Config': {u'Tty': False, u'Cmd': [u'/bin/bash'], u'Volumes': None, u'Domainname': u'', u'WorkingDir': u'', u'Image': u'5a2725191d75eb64e9b7c969cd23d8c67c6e8af9979e521a417bbfa34434fb83', u'Hostname': u'd6dcf178f680', u'StdinOnce': False, u'Labels': {u'build-date': u'20161102', u'vendor': u'CentOS', u'name': u'CentOS Base Image', u'license': u'GPLv2'}, u'AttachStdin': False, u'User': u'', u'Env': [u'PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin'], u'Entrypoint': None, u'OnBuild': None, u'AttachStderr': False, u'AttachStdout': False, u'OpenStdin': False}, u'Author': u'https://github.com/CentOS/sig-cloud-instance-images', u'GraphDriver': {u'Data': {u'DeviceName': u'docker-253:1-20984667-e3af0c61256f885331fb1a3adc27ea509a10ba9a0ba9175c1a149f81bddcd30d', u'DeviceSize': u'10737418240', u'DeviceId': u'2'}, u'Name': u'devicemapper'}, u'VirtualSize': 196509652, u'Os': u'linux', u'Architecture': u'amd64', u'ContainerConfig': {u'Tty': False, u'Cmd': [u'/bin/sh', u'-c', u'#(nop) ', u'CMD ["/bin/bash"]'], u'Volumes': None, u'Domainname': u'', u'WorkingDir': u'', u'Image': u'5a2725191d75eb64e9b7c969cd23d8c67c6e8af9979e521a417bbfa34434fb83', u'Hostname': u'd6dcf178f680', u'StdinOnce': False, u'Labels': {u'build-date': u'20161102', u'vendor': u'CentOS', u'name': u'CentOS Base Image', u'license': u'GPLv2'}, u'AttachStdin': False, u'User': u'', u'Env': [u'PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin'], u'Entrypoint': None, u'OnBuild': None, u'AttachStderr': False, u'AttachStdout': False, u'OpenStdin': False}, u'Size': 196509652, u'RepoDigests': [u'docker.io/centos@b2f9d1c0ff5f87a4743104d099a3d561002ac500db1b9bfa02a783a46e0d366c'], u'Id': u'0584b3d2cf6d235ee310cf14b54667d889887b838d3f3d3033acd70fc3c48b8a', u'RepoTags': [u'docker.io/centos:latest']}
_rhel_images = [{'VirtualSize': 192508958, 'Id': 'f98706e16e41e56c4beaeea9fa77cd00fe35693635ed274f128876713afc0a1e', 'Labels': {'vcs-ref': '7eeaf203cf909c2c056fba7066db9c1073a28d97', 'build-date': '2016-10-26T07:54:17.037911Z', 'release': '45', 'Release': '45', 'Build_Host': 'rcm-img01.build.eng.bos.redhat.com', 'distribution-scope': 'public', 'Name': 'rhel7/rhel', 'vcs-type': 'git', 'com.redhat.build-host': 'ip-10-29-120-48.ec2.internal', 'vendor': 'Red Hat, Inc.', 'authoritative-source-url': 'registry.access.redhat.com', 'com.redhat.component': 'rhel-server-docker', 'architecture': 'x86_64', 'BZComponent': 'rhel-server-docker', 'Vendor': 'Red Hat, Inc.', 'version': '7.3', 'Version': '7.3', 'Architecture': 'x86_64', 'name': 'rhel7/rhel'}, 'ParentId': '', 'RepoDigests': ['registry.access.redhat.com/rhel7@da8a3e9297da7ccd1948366103d13c45b7e77489382351a777a7326004b63a21'], 'RepoTags': ['registry.access.redhat.com/rhel7:latest'], 'Created': 1477483353, 'Size': 192508958}]
_multiple_images = [{u'Created': 1477483353, u'Labels': {u'com.redhat.component': u'rhel-server-docker', u'authoritative-source-url': u'registry.access.redhat.com', u'distribution-scope': u'public', u'Vendor': u'Red Hat, Inc.', u'Name': u'rhel7/rhel', u'Build_Host': u'rcm-img01.build.eng.bos.redhat.com', u'vcs-type': u'git', u'name': u'rhel7/rhel', u'vcs-ref': u'7eeaf203cf909c2c056fba7066db9c1073a28d97', u'release': u'45', u'Version': u'7.3', u'Architecture': u'x86_64', u'version': u'7.3', u'Release': u'45', u'vendor': u'Red Hat, Inc.', u'BZComponent': u'rhel-server-docker', u'build-date': u'2016-10-26T07:54:17.037911Z', u'com.redhat.build-host': u'ip-10-29-120-48.ec2.internal', u'architecture': u'x86_64'}, u'VirtualSize': 192508958, u'ParentId': u'', u'RepoTags': [u'registry.access.redhat.com/rhel7:latest'], u'RepoDigests': [u'registry.access.redhat.com/rhel7@da8a3e9297da7ccd1948366103d13c45b7e77489382351a777a7326004b63a21'], u'Id': u'f98706e16e41e56c4beaeea9fa77cd00fe35693635ed274f128876713afc0a1e', u'Size': 192508958}, {u'Created': 1456801947, u'Labels': {u'Vendor': u'Red Hat, Inc.', u'Name': u'rhel7/rhel', u'Build_Host': u'rcm-img02.build.eng.bos.redhat.com', u'Version': u'7.2', u'Architecture': u'x86_64', u'Release': u'46', u'BZComponent': u'rhel-server-docker', u'Authoritative_Registry': u'registry.access.redhat.com'}, u'VirtualSize': 203247534, u'ParentId': u'', u'RepoTags': [u'registry.access.redhat.com/rhel7:7.2-46'], u'RepoDigests': [u'registry.access.redhat.com/rhel7@6b76f95ab3ed949c2ca3110765f310bf8f4c9333394efba11727b14863c73f10'], u'Id': u'32f8a1d5f01956ca38f3887a83b1736da4f450d6132563c665bdc0a7a4684c4d', u'Size': 203247534}, {u'Created': 1455812978, u'Labels': {u'Vendor': u'Red Hat, Inc.', u'Name': u'rhel7/rhel', u'Build_Host': u'rcm-img01.build.eng.bos.redhat.com', u'Version': u'7.2', u'Architecture': u'x86_64', u'Release': u'44', u'BZComponent': u'rhel-server-docker', u'Authoritative_Registry': u'registry.access.redhat.com'}, u'VirtualSize': 203243624, u'ParentId': u'', u'RepoTags': [u'registry.access.redhat.com/rhel7:7.2-44'], u'RepoDigests': [u'registry.access.redhat.com/rhel7@af41b2f4e4fd2260247f1450b27da021d319ca4413e57ae833313398334d6e6f'], u'Id': u'da3ab9be742254035a2736da5d41c0a941b9a12ddd29f70b6129748e3749dd01', u'Size': 203243624}]


@unittest.skipIf(no_mock, "Mock not found")
class TestDockerBackend(unittest.TestCase):


    def test_get_images(self):
        db = DockerBackend()
        db.d.images = MagicMock(return_value=d_images)
        images = db.get_images()
        self.assertTrue(len(images) > 0)

    def test_inspect_image(self):
        db = DockerBackend()
        db._inspect_image = MagicMock(return_value=_rsyslog_image_inspect)
        img_obj = db.inspect_image('registry.access.redhat.com/rhel7/rsyslog:latest')
        self.assertIsNotNone(img_obj)

    def test_long_version(self):
        db = DockerBackend()
        db._inspect_image = MagicMock(return_value=_rsyslog_image_inspect)
        img_obj = db.inspect_image('registry.access.redhat.com/rhel7/rsyslog:latest')
        self.assertEqual(img_obj.long_version, "7.3-8")

    def test_inspect_container(self):
        db = DockerBackend()
        db._inspect_container = MagicMock(return_value=_rsyslog_container_inspect)
        con_obj = db.inspect_container('3e014f35a7d0')
        self.assertIsNotNone(con_obj)

    def test_container_obj_id(self):
        db = DockerBackend()
        db._inspect_container = MagicMock(return_value=_rsyslog_container_inspect)
        con_obj = db.inspect_container('3e014f35a7d0')
        self.assertEqual(con_obj.id, '3e014f35a7d0a22579cdad033edefd287fe01cac0e52dd68162e477fc150dc64')
        self.assertEqual(con_obj.image, '04f7e9543b939537000296e38c11d5d422653f74a6dd2a3905b931bb8f4c3265')

    def test_has_image_by_inspect(self):
        db = DockerBackend()
        db._inspect_image = MagicMock(return_value=_rsyslog_image_inspect)
        img_object = db.has_image('registry.access.redhat.com/rhel7/rsyslog:latest')
        self.assertEqual(img_object.id, '04f7e9543b939537000296e38c11d5d422653f74a6dd2a3905b931bb8f4c3265')

    def test_has_image_by_name(self):
        with patch('Atomic.util.image_by_name') as mockobj:
            mockobj.return_value = _centos_name_search
            db = DockerBackend()
            db.inspect_image = MagicMock(return_value=None)
            db._inspect_image = MagicMock(return_value=_centos_inspect_image)
            img_obj = db.has_image('centos')
            self.assertEqual(img_obj.id, '0584b3d2cf6d235ee310cf14b54667d889887b838d3f3d3033acd70fc3c48b8a')


    def test_has_image_not_found(self):
        with patch('Atomic.util.image_by_name') as mockobj:
            mockobj.return_value = []
            db = DockerBackend()
            db._inspect_image = MagicMock(return_value=None)
            self.assertIsNone(db.has_image('foobar'))

    def test_has_image_multiple(self):
        with patch('Atomic.util.image_by_name') as mockobj:
            mockobj.return_value = _multiple_images
            db = DockerBackend()
            db._get_images = MagicMock(return_value=d_images)
            db._inspect_image = MagicMock(return_value=None)
            self.assertRaises(ValueError, db.has_image, 'registry.access.redhat.com/rhel7')

o_images = [{'Labels': {u'com.redhat.component': u'rhel-server-docker', u'authoritative-source-url': u'registry.access.redhat.com', u'distribution-scope': u'public', u'Vendor': u'Red Hat, Inc.', u'Name': u'rhel7/rhel', u'Build_Host': u'rcm-img01.build.eng.bos.redhat.com', u'vcs-type': u'git', u'build-date': u'2016-10-26T07:54:17.037911Z', u'Release': u'45', u'Version': u'7.3', u'Architecture': u'x86_64', u'version': u'7.3', u'release': u'45', u'vendor': u'Red Hat, Inc.', u'BZComponent': u'rhel-server-docker', u'vcs-ref': u'7eeaf203cf909c2c056fba7066db9c1073a28d97', u'architecture': u'x86_64', u'com.redhat.build-host': u'ip-10-29-120-48.ec2.internal', u'name': u'rhel7/rhel'}, 'Names': [], 'Created': 1478630928, 'OSTree-rev': 'be942c330bde705f6dc5c82068927b671bbf1cf0c785ae7d93ae3d49f789fc2b', 'RepoTags': ['registry.access.redhat.com/rhel7:latest'], 'Id': u'da8a3e9297da7ccd1948366103d13c45b7e77489382351a777a7326004b63a21', 'ImageType': 'system', 'ImageId': u'da8a3e9297da7ccd1948366103d13c45b7e77489382351a777a7326004b63a21'}, {'Labels': {u'Vendor': u'Red Hat, Inc.', u'Name': u'rhel7/rhel', u'Build_Host': u'rcm-img01.build.eng.bos.redhat.com', u'Version': u'7.2', u'Architecture': u'x86_64', u'Release': u'44', u'BZComponent': u'rhel-server-docker', u'Authoritative_Registry': u'registry.access.redhat.com'}, 'Names': [], 'Created': 1478631019, 'OSTree-rev': 'f5c4a4dddaad670feb80aa272c6ea228831ee90f6dba93f0bbef246bfd96eb43', 'RepoTags': ['registry.access.redhat.com/rhel7:7.2-44'], 'Id': u'af41b2f4e4fd2260247f1450b27da021d319ca4413e57ae833313398334d6e6f', 'ImageType': 'system', 'ImageId': u'af41b2f4e4fd2260247f1450b27da021d319ca4413e57ae833313398334d6e6f'}, {'Labels': {u'Vendor': u'Red Hat, Inc.', u'Name': u'rhel7/rhel', u'Build_Host': u'rcm-img02.build.eng.bos.redhat.com', u'Version': u'7.2', u'Architecture': u'x86_64', u'Release': u'46', u'BZComponent': u'rhel-server-docker', u'Authoritative_Registry': u'registry.access.redhat.com'}, 'Names': [], 'Created': 1478631057, 'OSTree-rev': '6705ceedb07aad7040d2e7251117cb5d8a2b56750735c621b51f1969c7e55fbd', 'RepoTags': ['registry.access.redhat.com/rhel7:7.2-46'], 'Id': u'6b76f95ab3ed949c2ca3110765f310bf8f4c9333394efba11727b14863c73f10', 'ImageType': 'system', 'ImageId': u'6b76f95ab3ed949c2ca3110765f310bf8f4c9333394efba11727b14863c73f10'}, {'Labels': {}, 'Names': [], 'Created': 1478631019, 'OSTree-rev': 'c10f4558f1d0eddc51f0d060cfc99401963d8bcdc680ea972658560d4fcc63a3', 'RepoTags': ['<none>'], 'Id': '48ef8b1e90ce99f9716358e924be716c227b9b7fd55378cca04622f717acef55', 'ImageType': 'system', 'ImageId': '48ef8b1e90ce99f9716358e924be716c227b9b7fd55378cca04622f717acef55'}, {'Labels': {}, 'Names': [], 'Created': 1478630866, 'OSTree-rev': '3ee8339a98332077459ada306fc48e9bdcc664f9f06aae7d5be8ef8d8e77468a', 'RepoTags': ['<none>'], 'Id': '3690ec4760f95690944da86dc4496148a63d85c9e3100669a318110092f6862f', 'ImageType': 'system', 'ImageId': '3690ec4760f95690944da86dc4496148a63d85c9e3100669a318110092f6862f'}, {'Labels': {}, 'Names': [], 'Created': 1478630953, 'OSTree-rev': '54326755640d70b26b15d35295e4ffa7c01e9bc0a6b46049cbed8088a36dacc6', 'RepoTags': ['docker.io/library/busybox:latest'], 'Id': u'29f5d56d12684887bdfa50dcd29fc31eea4aaf4ad3bec43daf19026a7ce69912', 'ImageType': 'system', 'ImageId': u'29f5d56d12684887bdfa50dcd29fc31eea4aaf4ad3bec43daf19026a7ce69912'}, {'Labels': {}, 'Names': [], 'Created': 1478631057, 'OSTree-rev': '793efe11094a09d47b7e42f6de8689b2519bbb7eca98a8c2f48caca0db2be59f', 'RepoTags': ['<none>'], 'Id': 'f075d88cadb6e82d374f1dd04ca9e6e44a24bbb524cf5b235e92bffca89c04fa', 'ImageType': 'system', 'ImageId': 'f075d88cadb6e82d374f1dd04ca9e6e44a24bbb524cf5b235e92bffca89c04fa'}, {'Labels': {}, 'Names': [], 'Created': 1478630928, 'OSTree-rev': 'bd3d84f73d3424a61c5797688a2eea12e83b3a441c95448bfcf943cac9518571', 'RepoTags': ['<none>'], 'Id': '9b0dee6356a152d6cbac4adf6ce40ae4df40442d559931862ad65c78ed373979', 'ImageType': 'system', 'ImageId': '9b0dee6356a152d6cbac4adf6ce40ae4df40442d559931862ad65c78ed373979'}, {'Labels': {}, 'Names': [], 'Created': 1478630928, 'OSTree-rev': 'd10d6facaa596ea16ff7f30ce221669fce15a60ee643ad051a65f0d2ce855ec8', 'RepoTags': ['<none>'], 'Id': '972548a33962c6f466128fc90db5d224d8bb5f589ffd9a998a43f323c7111cf5', 'ImageType': 'system', 'ImageId': '972548a33962c6f466128fc90db5d224d8bb5f589ffd9a998a43f323c7111cf5'}, {'Labels': None, 'Names': [], 'Created': 1478630866, 'OSTree-rev': 'b42139867c4c55a28bee9d63801c3c892bfdf23fe59c4624a49f99b6aa138d1d', 'RepoTags': ['alpine:latest'], 'Id': u'1354db23ff5478120c980eca1611a51c9f2b88b61f24283ee8200bf9a54f2e5c', 'ImageType': 'system', 'ImageId': u'1354db23ff5478120c980eca1611a51c9f2b88b61f24283ee8200bf9a54f2e5c'}, {'Labels': {}, 'Names': [], 'Created': 1478630953, 'OSTree-rev': '5ba9ede4f40d0fb001906cedc45ae001015172042f50585924674f78e5b16ad0', 'RepoTags': ['<none>'], 'Id': '56bec22e355981d8ba0878c6c2f23b21f422f30ab0aba188b54f1ffeff59c190', 'ImageType': 'system', 'ImageId': '56bec22e355981d8ba0878c6c2f23b21f422f30ab0aba188b54f1ffeff59c190'}]


@unittest.skipIf(no_mock, "Mock not found")
class TestOSTreeBackend(unittest.TestCase):

    def test_get_images(self):
        pass

@unittest.skipIf(no_mock, "Mock not found")
class TestBackendUtils(unittest.TestCase):

    bu = BackendUtils()

    def test_get_backend_for_image_preferred(self):
        pass

    def test_get_backend_for_image(self):
        pass

if __name__ == '__main__':
    unittest.main()



from . import Atomic
from . import mount
from . import util
from datetime import datetime
import os
from shutil import rmtree
import json
import sys
from Atomic.backendutils import BackendUtils

def cli(subparser):
    # atomic scan
    scanners = util.get_scanners()
    scanp = subparser.add_parser(
        "scan", help=_("scan an image or container for CVEs"),
        epilog="atomic scan <input> scans a container or image for CVEs")
    scanp.set_defaults(_class=Scan, func='scan')
    scan_group = scanp.add_mutually_exclusive_group()
    scanp.add_argument("scan_targets", nargs='*', help=_("container image"))
    scanp.add_argument("--scanner", choices=[x['scanner_name'] for x in scanners], default=None, help=_("define the intended scanner"))
    scanp.add_argument("--scan_type", default=None, help=_("define the intended scan type"))
    scanp.add_argument("--list", action='store_true', default=False, help=_("List available scanners"))
    disp_group = scanp.add_mutually_exclusive_group()
    disp_group.add_argument("--verbose", action='store_true', default=False, help=_("Show more output from scanning container"))
    disp_group.add_argument("--json", action='store_true', default=False, help=_("Output results in JSON format"))
    scan_group.add_argument("--rootfs", nargs='?', default=[], action='append', help=_("Rootfs path to scan"))
    scan_group.add_argument("--all", default=False, action='store_true', help=_("scan all images (excluding intermediate layers) and containers"))
    scan_group.add_argument("--images", default=False, action='store_true', help=_("scan all images (excluding intermediate layers)"))
    scan_group.add_argument("--containers", default=False, action='store_true', help=_("scan all containers"))


class Scan(Atomic):
    """
    Scan class that can generically work any scanner
    """


    def __init__(self):
        super(Scan, self).__init__()
        self.scan_dir = None
        self.scan_list = []
        self.rootfs_paths = []
        self.cur_time = datetime.now().strftime('%Y-%m-%d-%H-%M-%S-%f')
        self.chroot_dir = '/run/atomic/{}'.format(self.cur_time)
        self.results_dir = None
        self.scan_content = {}
        self.atomic_config = util.get_atomic_config()
        self.scanners = util.get_scanners()
        self.debug = False
        self.rootfs_mappings = {}
        self.scanner = None
        self.mount_paths = {}

    def get_scanners_list(self):
        return self.scanners

    def scan(self):
        def get_scan_info(scanner, scan_type):
            for i in self.scanners:
                if i['scanner_name'] == scanner:
                    for x in i['scans']:
                        if x['name'] == scan_type:
                            return i['image_name'], x['args'], i.get('custom_args')

        def set_scanner():
            # If no scanners are defined
            if len(self.scanners) == 0:
                raise ValueError("No scanners are configured for your system.")

            if self.args.scanner:
                self.scanner = self.args.scanner
            elif len(self.scanners) == 1:
                self.scanner = self.scanners[0]['scanner_name']
            # Use default
            else:
                self.scanner = self.atomic_config['default_scanner']

            if self.scanner is None:
                raise ValueError("You must specify a scanner (--scanner) or set a default in /etc/atomic.conf")

            # Check if the scanner name is valid
            if self.scanner not in [x['scanner_name'] for x in self.scanners]:
                raise ValueError("Unknown scanner '{}' defined in {}".format(self.scanner, util.ATOMIC_CONF))

        if self.args.debug:
            self.debug = True

        # Set debug bool
        self.set_debug()

        # Show list of scanners and exit
        if self.args.list:
            self.print_scan_list()

        set_scanner()

        scan_type = self.get_scan_type()

        # Load the atomic config file and check scanner settings
        yaml_error = "The image name or scanner arguments for '{}' is not " \
                     "defined in /etc/atomic.conf".format(self.scanner)

        scanner_image_name, scanner_args, custom_args = get_scan_info(self.scanner, scan_type)

        if not isinstance(scanner_args, list):
            raise ValueError("The scanner arguments for {} must be in list"
                             " ([]) form.".format(self.scanner))

        if None in [scanner_image_name, scanner_args]:
            raise ValueError(yaml_error)

        self.results_dir = os.path.join(self.results, self.scanner, self.cur_time)

        # Create the input directory
        if not os.path.exists(self.chroot_dir):
            os.makedirs(self.chroot_dir)
        if self.debug:
            util.write_out("Created {}".format(self.chroot_dir))

        if len(self.args.rootfs) == 0:
            self.scan_list = self._get_scan_list()
            security_args = []
        else:
            # Check to make sure all the chroots provided are legit
            for _path in self.args.rootfs:
                if not os.path.exists(_path):
                    raise ValueError("The path {} does not exist".format(_path))
            self.setup_rootfs_dirs()
            # Have to disable SELinux checking of scanning not docker images
            security_args = [ "--security-opt", "label:disable" ]

        # Create the output directory
        os.makedirs(self.results_dir)

        docker_args = ['docker', 'run', '-t', '--rm', '-v', '/etc/localtime:/etc/localtime',
                       '-v', '{}:{}'.format(self.chroot_dir, '/scanin'), '-v',
                       '{}:{}:rw,Z'.format(self.results_dir, '/scanout')]

        # Assemble the cmd line for the scan
        scan_cmd = docker_args + security_args
        if custom_args is not None:
            scan_cmd = scan_cmd + custom_args
        scan_cmd = scan_cmd + [scanner_image_name] + scanner_args
        scan_cmd = self.sub_env_strings(" ".join(scan_cmd))

        # Show the command being run
        if self.useTTY and not self.args.json:
            util.write_out(scan_cmd)

        # Show stdout from container if --debug or --verbose
        stdout = None if (self.args.verbose or self.args.debug) else open(os.devnull, 'w')

        try:
            if len(self.args.rootfs) == 0:
                # mount all the rootfs
                self._mount_scan_rootfs(self.scan_list)

                if self.debug:
                    util.write_out("Creating the output dir at {}".format(self.results_dir))

            # do the scan
            util.check_call(scan_cmd, stdout=stdout, env=self.cmd_env())

            # output results
            if self.useTTY:
                self.output_results()

            # record environment
            self.record_environment()

            self.write_persistent_data()
        finally:
            # unmount all the rootfs
            self._unmount_rootfs_in_dir()

        if not self.useTTY:
            return (json.dumps(self.get_scan_data()))


        for json_file in self._get_json_files():
            json_results = json.load(open(json_file))
            if json_results['Successful'].upper() != 'TRUE':
                return 1
        return 0

    def _get_scan_list(self):
        beu = BackendUtils()
        if self.args.images:
            scan_list = beu.get_images()
        elif self.args.containers:
            scan_list = beu.get_containers()
        elif self.args.all:
            scan_list = beu.get_images() + beu.get_containers()
        else:
            scan_list = []
            for scan_target in self.args.scan_targets:
                try:
                    # get_backend_and_container throws a ValueError when it cannot find anything
                    _, scan_obj = beu.get_backend_and_container_obj(scan_target)
                except ValueError:
                    try:
                        # get_backend_and_image throws a ValueError when it cannot find anything
                        _, scan_obj = beu.get_backend_and_image_obj(scan_target)
                    except ValueError:
                        raise ValueError("Unable to locate the container or image '{}'".format(scan_target))
                scan_list.append(scan_obj)
        return scan_list

    def _get_roots_path_from_bind_name(self, in_bind_name):
        for _path, bind_path in self.rootfs_mappings.items():
            if bind_path == os.path.basename(os.path.split(in_bind_name)[0]):
                return _path

    def get_scan_data(self):
        results = []
        json_files = self._get_json_files()
        for json_file in json_files:
            results.append(json.load(open(json_file)))
        return results

    def _mount_scan_rootfs(self, scan_list):
        for docker_object in scan_list:
            mount_path = os.path.join(self.chroot_dir, docker_object.id.replace("/", "_"))
            os.mkdir(mount_path)
            if self.debug:
                util.write_out("Created {}".format(mount_path))
            new_mount_path = self.mount(mountpoint=mount_path, image=docker_object.id)
            if new_mount_path != mount_path:
                rmtree(mount_path)
                os.symlink(new_mount_path, mount_path)
            self.mount_paths[new_mount_path.rstrip('/')] = docker_object.id
            if self.debug:
                util.write_out("Mounted {} to {}".format(docker_object.id, mount_path))

    def _unmount_rootfs_in_dir(self):
        for rootfs_dir in self.get_rootfs_paths():
            if len(self.args.rootfs) == 0:
                try:
                    self.unmount(rootfs_dir)
                except (ValueError, mount.MountError):
                    pass
            else:
                # Clean up bind mounts if the chroot feature is used
                mcmd = ['umount', rootfs_dir]
                util.check_call(mcmd)

            # Clean up temporary containers
            if not self.debug:
                # Remove the temporary container dirs
                rmtree(rootfs_dir)
            else:
                util.write_out("Unmounted {}".format(rootfs_dir))
        if not self.debug:
            rmtree(self.chroot_dir)

    def get_rootfs_paths(self):
        """
        Returns the list of rootfs paths (not fully qualified)
        :return: list
        """
        return self.mount_paths.keys()

    def output_results(self):
        """
        Write results of the scan to stdout
        :return: None
        """
        json_files = self._get_json_files()
        for json_file in json_files:
            json_results = json.load(open(json_file))
            if self.args.json:
                util.output_json(json_results)
            else:
                uuid = next(self.mount_paths[x] for x in self.mount_paths.keys() if
                            os.path.basename(json_results['UUID'].rstrip('/')) in x) if len(self.args.rootfs) == 0 \
                    else self._get_roots_path_from_bind_name(json_file)

                name1 = uuid if len(self.args.rootfs) > 1 else self._get_input_name_for_id(uuid)
                if len(self.args.rootfs) == 0 and not self._is_iid(uuid):
                    name2 = uuid[:15]
                else:
                    # Containers do not have repo names
                    if len(self.args.rootfs) == 0 and uuid not in [x['Id'] for x in self.get_containers()]:
                        name2 = self._get_repo_names(uuid)
                    else:
                        name2 = uuid[:15]
                util.write_out("\n{} ({})\n".format(name1, name2))
                if json_results['Successful'].upper() == "TRUE":
                    if 'Custom' in json_results:
                        self._output_custom(json_results['Custom'], 3)
                    if 'Vulnerabilities' in json_results and len(json_results['Vulnerabilities']) > 0:
                        util.write_out("The following issues were found:\n")
                        for vul in json_results['Vulnerabilities']:
                            if 'Title' in vul:
                                util.write_out("{}{}".format(' ' * 5, vul['Title']))
                            if 'Severity' in vul:
                                util.write_out("{}Severity: {}".format(' ' * 5, vul['Severity']))
                            if 'Custom' in vul.keys() and len(vul['Custom']) > 0:
                                custom_field = vul['Custom']
                                self._output_custom(custom_field, 7)
                            util.write_out("")
                    elif 'Results' in json_results and len(json_results['Results']) > 0:
                        util.write_out("The following results were found:\n")
                        for result in json_results['Results']:
                            if 'Custom' in result.keys() and len(result['Custom']) > 0:
                                custom_field = result['Custom']
                                self._output_custom(custom_field, 7)
                        util.write_out("")
                    else:
                        util.write_out("{} passed the scan".format(self._get_input_name_for_id(uuid)))
                else:
                    util.write_out("{}{} is not supported for this scan."
                                   .format(' ' * 5, self._get_input_name_for_id(uuid)))
        if not self.args.json:
            util.write_out("\nFiles associated with this scan are in {}.\n".format(self.results_dir))

    def _output_custom(self, value, indent):
        space = ' ' * indent
        next_indent = indent + 2
        if isinstance(value, dict):
            for x in value:
                if isinstance(value[x], dict):
                    util.write_out("{}{}:".format(space, x))
                    self._output_custom(value[x], next_indent)
                elif isinstance(value[x], list):
                    util.write_out("{}{}:".format(space, x))
                    self._output_custom(value[x], next_indent)
                else:
                    util.write_out("{}{}: {}".format(space, x, value[x]))
        elif isinstance(value, list):
            for x in value:
                if isinstance(x, dict):
                    self._output_custom(x, next_indent)
                elif isinstance(x, list):
                    self._output_custom(x, next_indent)
                else:
                    util.write_out('{}{}'.format(space, x))

    def _get_json_files(self):
        json_files = []
        for files in os.walk(self.results_dir):
            for jfile in files[2]:
                if jfile == 'json':
                    json_files.append(os.path.join(files[0], jfile))
        return json_files


    def _get_object_by_id(self, iid):
        scan_obj = next((x for x in self.scan_list if x.id == iid))
        return scan_obj

    def _get_input_name_for_id(self, iid):
        if len(self.args.rootfs) > 0:
            return iid
        else:
            return self._get_object_by_id(iid).input_name

    def _is_iid(self, input_name):
        scan_obj = next((x for x in self.scan_list if x.id == input_name))
        if scan_obj.id == scan_obj.input_name:
            return True
        return False

    def _get_repo_names(self, docker_id):
        _match = next((x for x in self.get_images() if x['Id'] == docker_id), None)
        if _match is None:
            _match = next((x for x in self.get_containers() if x['Id'] == docker_id), None)
        if'<none>' in _match['RepoTags'][0]:
            return docker_id[:15]
        else:
            return ', '.join(_match['RepoTags'])

    def record_environment(self):
        """
        Grabs a "snapshot" the results of docker info and inspect results for
        all images and containers.  Write it to results_dir/environment.json
        :return: None
        """

        environment = {}
        environment['info'] = self.d.info()
        environment['images'] = []
        for iid in [x['Id'] for x in self.get_images()]:
            environment['images'].append(self._inspect_image(image=iid))

        environment['containers'] = []
        for cid in [x['Id'] for x in self.get_containers()]:
            environment['containers'].append(self._inspect_container(name=cid))

        with open(os.path.join(self.results_dir, 'environment.json'), 'w') as f:
            json.dump(environment, f, indent=4, separators=(',', ': '))

    def get_scan_type(self):
        default_scan_type = None
        scan_types = []
        for i in self.scanners:
            if i['scanner_name'] == self.scanner:
                default_scan_type = i.get('default_scan')
                if self.args.scan_type is None and default_scan_type is None:
                    raise ValueError("No scan type was given and there is no "
                                     "default scan type defined for '{}'".format(self.args.scanner))
                for x in i['scans']:
                    scan_types.append(x['name'])
        if default_scan_type not in scan_types:
            raise ValueError("The default scan type '{}' is not defined as a valid scan type for "
                             "the scanner '{}'".format(default_scan_type, self.scanner))
        if self.args.scan_type is None:
            return default_scan_type
        elif self.args.scan_type in scan_types:
            return self.args.scan_type
        else:
            raise ValueError("Unable to find the scan type '{}' for '{}'.".
                             format(self.args.scan_type, self.scanner))

    def print_scan_list(self):
        if len(self.scanners) == 0:
            util.write_out("There are no scanners configured for this system.")
            sys.exit(0)
        default_scanner = self.atomic_config['default_scanner']
        if default_scanner is None:
            default_scanner = ''
        for scanner in self.scanners:
            scanner_name = scanner['scanner_name']
            df = '* ' if scanner_name == default_scanner else ''
            default_scan_type = scanner.get('default_scan')
            if default_scan_type is None:
                raise ValueError("Invalid configuration file: At least one scan type must be "
                                 "declared as the default for {}.".format(scanner_name))
            util.write_out("Scanner: {} {}".format(scanner_name, df))
            util.write_out("{}Image Name: {}".format(" " * 2, scanner['image_name']))
            for scan_type in scanner['scans']:
                df = '* ' if default_scan_type == scan_type['name'] else ''
                util.write_out("{}Scan type: {} {}".format(" " * 5, scan_type['name'], df))
                util.write_out("{}Description: {}\n".format(" " * 5, scan_type['description']))
        util.write_out("\n* denotes defaults")
        sys.exit(0)

    def mount(self, mountpoint, image):
        m = mount.Mount()
        m.set_args(self.args)
        m.mountpoint = mountpoint
        m.image = image
        m.shared = True
        m.mount(best_mountpoint_for_storage=True)
        return m.mountpoint

    def unmount(self, mountpoint):
        m = mount.Mount()
        m.set_args(self.args)
        m.mountpoint = mountpoint
        m.unmount()

    def setup_rootfs_dirs(self):
        for _dir in self.args.rootfs:
            bind_dir = _dir.replace("/", "_")
            chroot_scan_dir = os.path.join(self.chroot_dir, bind_dir)
            os.mkdir(chroot_scan_dir)
            mcmd = ['mount', '-o', 'ro,bind', _dir, chroot_scan_dir]
            util.check_call(mcmd)
            self.rootfs_mappings[_dir] = bind_dir

    def get_persist_data(self, json_results, json_file):
        persist = {}
        if json_results.get('Successful').upper() == "FALSE":
            return {}
        persist['UUID'] = json_results['UUID'].replace("/scanin/", "")
        persist['Scanner'] = json_results['Scanner']
        persist['Time'] = json_results['Time']
        persist['Scan Type'] = json_results['Scan Type']
        if 'Vulnerabilities' in json_results and len(json_results['Vulnerabilities']) > 0:
            persist["Vulnerable"] =  True
        elif json_results.get('Vulnerable') is True:
            persist["Vulnerable"] =  True
        else:
            persist["Vulnerable"] =  False
        persist['json_file'] = json_file
        return persist


    def write_persistent_data(self):
        new_data = dict()
        json_files = self._get_json_files()
        for json_file in json_files:
            json_results = json.load(open(json_file))
            uuid = os.path.basename(json_results['UUID']) if len(self.args.rootfs) == 0 \
                else self._get_roots_path_from_bind_name(json_file)
            new_data[uuid] = self.get_persist_data(json_results, json_file)
        summary_file = os.path.join(self.results, "scan_summary.json")
        if not os.path.exists(summary_file):
            persistent_data = new_data
        else:
            persistent_data = json.loads(open(summary_file, "r").read())
            iids = [x['Id'] for x in self.get_images()]
            cids = [x['Id'] for x in self.get_containers()]
            for uuid in new_data.keys():
                if len(new_data[uuid]) > 0:
                    persistent_data[uuid] = new_data[uuid]

            # Clean up old data
            for uuid in list(persistent_data):
                if uuid not in iids and uuid not in cids:
                    del persistent_data[uuid]

        with open(summary_file, 'w') as f:
            json.dump(persistent_data, f, indent=4)



from Atomic.util import output_json
from Atomic.client import no_shaw

class Layer(object):
    def __init__(self, img_input):
        self.id = None
        self.name = None
        self.version = None
        self.release = None
        self.repotags = None
        self.parent = None
        self.remote = False
        self.digest = None
        self.backend = None

        if type(img_input) is dict:
            pass
        else:
            self._instantiate_from_image_object(img_input)

    def _instantiate_from_image_object(self, img_obj):
        self.id = img_obj.id
        self.name = img_obj.get_label('Name') or img_obj.name or img_obj.image
        self.remote = img_obj.remote
        self.version = img_obj.version
        self.release = img_obj.release
        self.repotags = img_obj.repotags
        # This needs to be resolved for future docker versions
        self.parent = img_obj.parent
        self.digest = img_obj.digest
        self.backend = img_obj.backend
        return self

    def _instantiate_from_dict(self):
        return self

    def __eq__(self, other):
        if self.long_version == other.long_version:
            return True
        return False

    def __ne__(self, other):
        if self.long_version != other.long_version:
            return True
        return False

    def dump(self):
        # helper function to dump out known variables/values in pretty-print style
        class_vars = dict(vars(self))
        foo = {x: class_vars[x] for x in class_vars if not callable(getattr(self, x)) and not x.startswith('__')
               and not x.endswith('_backend')}
        output_json(foo)

    @property
    def long_version(self):
        _version = ""
        if self.version:
            _version += "{}".format(self.version)
        if self.release:
            if self.version:
                _version += "-"
            _version += "{}".format(self.release)
        if not _version:
            return no_shaw(self.id or self.digest)
        return _version



#! /usr/bin/python3 -Es
import sys
import dbus
import time
from Atomic import util
from slip.dbus import polkit
import dbus.service
import dbus.mainloop.glib

ATOMIC_CONFIG = util.get_atomic_config()
_storage = ATOMIC_CONFIG.get('default_storage', "docker")

class AtomicDBus (object):
    def __init__(self):
        self.bus = dbus.SystemBus()
        self.dbus_object = self.bus.get_object("org.atomic", "/org/atomic/object")

    @polkit.enable_proxy
    def ContainersList(self):
        return self.dbus_object.ContainersList(dbus_interface="org.atomic")

    @polkit.enable_proxy
    def ContainersDelete(self, containers, containers_all=False, force=False, storage=_storage):
        if not isinstance(containers, (list, tuple)):
            containers = [ containers ]
        return self.dbus_object.ContainersDelete(containers, containers_all, force, storage, dbus_interface="org.atomic" )

    @polkit.enable_proxy
    def ContainersTrim(self):
        return self.dbus_object.ContainersTrim(dbus_interface="org.atomic", timeout = 2147400)

    @polkit.enable_proxy
    def Diff(self, first, second, rpms=False, no_files=False, names_only=False, diff_keywords=None, metadata=False):
        assert(isinstance(diff_keywords, list))
        if diff_keywords is None:
            diff_keywords = []
        return self.dbus_object.Diff(first, second, rpms, no_files, names_only, diff_keywords, metadata, dbus_interface="org.atomic", timeout = 2147400)

    @polkit.enable_proxy
    def Stop(self, name):
        return self.dbus_object.Stop(name, dbus_interface="org.atomic", timeout = 2147400)

    @polkit.enable_proxy
    def StorageExport(self, graph, export_location, force):
        self.dbus_object.StorageExport(graph, export_location, force, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def StorageImport(self, graph, import_location):
        self.dbus_object.StorageImport(graph, import_location, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def StorageModify(self, devices, driver):
        self.dbus_object.StorageModify(devices, driver, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def StorageReset(self):
        self.dbus_object.StorageReset(dbus_interface="org.atomic")

    @polkit.enable_proxy
    def AsyncScan(self, scan_targets, scanner, scan_type, rootfs, _all, images, containers):
        token = self.dbus_object.ScheduleScan(scan_targets, scanner, scan_type, rootfs, _all, images, containers, dbus_interface="org.atomic", timeout = 2147400)
        while(True):
            ret = self.dbus_object.GetScanResults(token, dbus_interface="org.atomic")
            if ret:
                break
            time.sleep(1)

        return ret

    @polkit.enable_proxy
    def ImagesDelete(self, images, force=False, remote=False):
        if not isinstance(images, (list, tuple)):
            images = [ images ]
        return self.dbus_object.ImagesDelete(images, remote, force, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def ImagesHelp(self, image):
        return self.dbus_object.ImagesHelp(image, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def ImagesInfo(self, image, remote=False):
        return self.dbus_object.ImagesInfo(image, remote, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def ImagesList(self):
        return self.dbus_object.ImagesList(dbus_interface="org.atomic")

    @polkit.enable_proxy
    def ImagesPrune(self):
        return self.dbus_object.ImagesPrune(dbus_interface="org.atomic")

    # The ImagesPull method will pull the specified image
    @polkit.enable_proxy
    def ImagePull(self, image, storage="docker", reg_type=""):
        return self.dbus_object.ImagePull(image, storage, reg_type, dbus_interface="org.atomic", timeout = 2147400)

    def ImageUpdate(self, image, force=False):
        return self.dbus_object.ImageUpdate(image, force, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def ImageVersion(self, image, recurse=False):
        return self.dbus_object.ImageVersion(image, recurse, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def Install(self, image, name='', system=False, remote=False, storage='', user=False, setvalues=''):
        if not name:
            name = image
        if not setvalues:
            setvalues = []
        if not isinstance(setvalues, (list, tuple)):
            setvalues = [ setvalues ]
        return self.dbus_object.Install(image, name, system, remote, storage, user, setvalues, dbus_interface="org.atomic", timeout = 2147400)

    @polkit.enable_proxy
    def MountImage(self, src, dest, options="", live=False, shared=False):
        ret = self.dbus_object.MountImage(src, dest, options, live, shared, dbus_interface="org.atomic")
        return ret

    # The Run method will create and run a container on the specified image
    @polkit.enable_proxy
    def Run(self, image, name=None, spc=False, detach=False, command=None):
        if not name:
            name = image
        if not command:
            command = []
        if not isinstance(command, (list, tuple)):
            command = [ command ]
        return self.dbus_object.Run(image, name, spc, detach, command, dbus_interface="org.atomic", timeout = 2147400)

    @polkit.enable_proxy
    def Scan(self, scan_targets, scanner, scan_type, rootfs, _all, images, containers):
        return self.dbus_object.Scan(scan_targets, scanner, scan_type, rootfs, _all, images, containers, dbus_interface="org.atomic", timeout = 2147400)

    @polkit.enable_proxy
    def ScanList(self):
        ret = self.dbus_object.ScanList(dbus_interface="org.atomic")
        return ret

    @polkit.enable_proxy
    def Sign(self, images, sign_by, signature_path = "", gnupghome=""):
        if not isinstance(images, (list, tuple)):
            images = [ images ]
        return self.dbus_object.Sign(images, sign_by, signature_path, gnupghome, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def Top(self, containers=None, options=""):
        if not containers:
            containers=[]
        if not isinstance(containers, (list, tuple)):
            containers = [ containers ]
        return self.dbus_object.Top(containers, options, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def TrustAdd(self, registry, trusttype="reject", pubkeys="", keytype="GPGKeys", sigstore="", sigstoretype="web"):
        return self.dbus_object.TrustAdd(registry, trusttype, pubkeys, keytype, sigstore, sigstoretype, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def TrustDefaultPolicy(self, policy):
        return self.dbus_object.TrustDefaultPolicy(policy, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def TrustDelete(self, registry, trusttype="None"):
        return self.dbus_object.TrustDelete(registry, trusttype, dbus_interface="org.atomic")

    @polkit.enable_proxy
    def TrustShow(self):
        return self.dbus_object.TrustShow(dbus_interface="org.atomic")

    @polkit.enable_proxy
    def Uninstall(self, image, name=None, force=False, storage=None, extra_args=None):
        if not name:
            name = image
        if not extra_args:
            extra_args = []
        if not isinstance(extra_args, (list, tuple)):
            extra_args = [ extra_args ]
        return self.dbus_object.Install(image, name=name, force=force, storage=storage, extra_args=extra_args, dbus_interface="org.atomic", timeout = 2147400)

    @polkit.enable_proxy
    def UnmountImage(self, dest):
        ret = self.dbus_object.UnmountImage(dest, dbus_interface="org.atomic")
        return ret

    @polkit.enable_proxy
    def Verify(self, image):
        ret = self.dbus_object.Verify(image, dbus_interface="org.atomic")
        return ret

    @polkit.enable_proxy
    def vulnerable(self):
        return self.dbus_object.VulnerableInfo(dbus_interface="org.atomic")

    @polkit.enable_proxy
    def GetScanResultsById(self, iid):
        return self.dbus_object.GetScanResultsById(iid, dbus_interface="org.atomic")


#For outputting the list of scanners
def print_scan_list(all_scanners):
    if len(all_scanners) == 0:
        util.write_out("There are no scanners configured for this system.")
        sys.exit(0)
    default_scanner = (util.get_atomic_config())['default_scanner']
    if default_scanner is None:
        default_scanner = ''
    for scanner in all_scanners:
        scanner_name = scanner['scanner_name']
        df = '* ' if scanner_name == default_scanner else ''
        default_scan_type = scanner.get('default_scan')
        if default_scan_type is None:
            raise ValueError("Invalid configuration file: At least one scan type must be "
                                 "declared as the default for {}.".format(scanner_name))
        util.write_out("Scanner: {} {}".format(scanner_name, df))
        util.write_out("{}Image Name: {}".format(" " * 2, scanner['image_name']))
        for scan_type in scanner['scans']:
            df = '* ' if default_scan_type == scan_type['name'] else ''
            util.write_out("{}Scan type: {} {}".format(" " * 5, scan_type['name'], df))
            util.write_out("{}Description: {}\n".format(" " * 5, scan_type['description']))
        util.write_out("\n* denotes defaults")
        sys.exit(0)

def is_number(var):
    try:
        int(var)
        return True
    except ValueError:
        return False

def convert_str(val):
    if val in ("True", "False"):
        return val
    if is_number(val):
        return val
    if val[0] == "[":
        return val
    return "\"%s\"" % val

if __name__ == "__main__":
    dbus_proxy = AtomicDBus()
    cmd="dbus_proxy.%s(" % sys.argv[1]

    if len(sys.argv[2:]) > 0:
        cmd+=convert_str(sys.argv[2])

    for i in sys.argv[3:]:
        cmd+=","
        cmd+=convert_str(i)
    cmd+=")"
    print(cmd)
    try:
        s = eval(cmd) # pylint: disable=eval-used
        if s:
            print(s)
    except dbus.exceptions.DBusException as e:
        print(e.get_dbus_message())
        sys.exit(-1)



from . import util
from . import Atomic
import os
import argparse
import json
import yaml
import requests
import collections
from base64 import b64encode,b64decode
import tempfile

TRANSPORT_TYPES = ["docker", "atomic", "dir"]

def cli(subparser):
    # atomic trust
    trustp = subparser.add_parser("trust",
                                  help=_("Manage system container trust policy"),
                                  epilog="Manages the trust policy of the host system. "
                                         "Trust policy describes a registry scope "
                                         "that must be signed by public keys.")
    commonp = argparse.ArgumentParser(add_help=False)
    registry_help="""Registry to manage trust policy for, REGISTRY[/REPOSITORY].
                     Trust policy allows for nested scope.
                     Example: registry.example.com defines trust for entire registry.
                     Example: registry.example.com/acme defines trust for specific repository."""
    sigstore_help="""Signature server type (default: web)
                     local: local file
                     web: remote web server
                     atomic: openshift-based atomic registry"""
    commonp.add_argument("-k", "--pubkeys", nargs='?', default=[],
                         action="append", dest="pubkeys",
                         help=_("Local path, URL of public key(s) or local user GPG "
                                "keyring ID to trust for TARGET. "
                                "Keys are parsed and encoded into policy.json. "
                                "May used multiple times to define multiple public keys. "
                                "File references must exist before using this command."))
    commonp.add_argument("-f", "--pubkeysfile", nargs='?', default=[],
                         action="append", dest="pubkeysfile",
                         help=_("Path of installed public key(s) to trust for TARGET. "
                                "Absolute path to keys is added to policy.json. "
                                "May used multiple times to define multiple public keys. "
                                "File(s) must exist before using this command."))
    commonp.add_argument("--sigstoretype", dest="sigstoretype", default="web",
                         choices=['atomic', 'local', 'web'],
                         help=sigstore_help)
    commonp.add_argument("-t", "--type", dest="trust_type", default="signedBy",
                         choices=['insecureAcceptAnything', 'reject', 'signedBy'],
                         help="Trust type (default: signedBy)")
    commonp.add_argument("--keytype", dest="keytype", default="GPGKeys",
                         help="Public key type (default: GPGKeys)")
    commonp.add_argument("-s", "--sigstore", dest="sigstore",
                         help=_("URL and path of remote signature server, "
                                "https://sigstore.example.com/signatures. "
                                "Ignored with 'atomic' sigstoretype."))
    commonp.add_argument("registry",
                         help=registry_help)
    subparsers = trustp.add_subparsers()
    addp = subparsers.add_parser("add", parents=[commonp],
                                 help="Add a new trust policy for a registry")
    addp.set_defaults(_class=Trust, func="add")
    defaultp = subparsers.add_parser("default",
                                 help="Modify default trust policy")
    defaultp.add_argument("default_policy", choices=["reject", "accept"],
                                 help="Default policy action")
    defaultp.set_defaults(_class=Trust, func="modify_default")
    deletep = subparsers.add_parser("delete",
                                    help="Delete a trust policy for a registry")
    deletep.add_argument("registry",
                         help=registry_help)
    deletep.add_argument("--sigstoretype", dest="sigstoretype", default="web",
                         choices=['atomic', 'local', 'web'],
                         help=sigstore_help)
    deletep.add_argument("--save-sigstore", dest="save", default=True, action="store_false",
                         help="Do not remove the registry sigstore configuration")
    deletep.set_defaults(_class=Trust, func="delete")
    showp = subparsers.add_parser("show",
                                  help="Display trust policy for the system")
    showp.add_argument('--raw', action='store_true', help="Output raw policy file")
    showp.add_argument('-j', '--json', action='store_true', help="Output as json")
    showp.set_defaults(_class=Trust, func="show")
    resetp = subparsers.add_parser("reset", help="Reset trust policy")
    resetp.set_defaults(_class=Trust, func="reset")

class Args():
    def __init__(self):
        self.debug = None
        self.assumeyes = None
        self.pubkeysfile = None


class Trust(Atomic):
    def __init__(self, policy_filename="/etc/containers/policy.json"):
        """
        :param policy_filename: override policy filename
        """
        super(Trust, self).__init__()
        self.policy_filename = os.environ.get('TRUST_POLICY', policy_filename)
        self.atomic_config = util.get_atomic_config()
        self.set_args(Args())

    def add(self, registry=None, pubkeys=None, pubkeysfile=None, sigstore=None, sigstoretype=None, keytype=None, trust_type=None):
        """
        Add trust to policy.json file and optionally update registries.d registry configuration
        :param sigstoretype: string, human-readable sigstore type, one of "atomic", "web", "local"
        :param registry: the registry[/repo] to add policy for
        :param pubkeys: list of pubkeys to encode in policy.json for trust_type "signedBy"
        :param pubkeysfile: list of pubkey paths in policy.json for trust_type "signedBy"
        :param keytype: string, "GPGKeys"
        :param trust_type: string, one of "signedBy", "insecureAcceptAnything", "reject"
        :param sigstore: string, URL of signature server
        """
        if registry is None:
            registry=self.args.registry
        if pubkeys is None:
            pubkeys=self.args.pubkeys
        if pubkeysfile is None:
            pubkeysfile=self.args.pubkeysfile
        if sigstoretype is None:
            sigstoretype=self.args.sigstoretype
        if keytype is None:
            keytype=self.args.keytype
        if trust_type is None:
            trust_type=self.args.trust_type
        if sigstore is None:
            sigstore=self.args.sigstore
        sstype = self.get_sigstore_type_map(sigstoretype)

        util.is_valid_image_uri(registry)
        mode = "r+" if os.path.exists(self.policy_filename) else "w+"
        with open(self.policy_filename, mode) as policy_file:
            if mode == "r+":
                policy = json.load(policy_file)
                policy = self.check_policy(policy, sstype)
                if registry in policy["transports"][sstype]:
                    if not self.args.assumeyes:
                        confirm = util.input("Trust policy already defined for %s:%s\nDo you want to overwrite? (y/N) " % (sigstoretype, registry))
                        if not "y" in confirm.lower():
                            exit(0)
            else:
                policy = self.default_policy_file
                policy["transports"][sstype] = {}

            payload = []
            if pubkeys:
                for k in pubkeys:
                    payload.append({ "type": trust_type, "keyType": keytype, "keyData": self.get_pubkey_data(k) })
            if pubkeysfile:
                for f in pubkeysfile:
                    if not os.path.exists(f):
                        raise ValueError("The public key file %s was not found. This file must exist to proceed." % f)
                    else:
                        payload.append({ "type": trust_type, "keyType": keytype, "keyPath": os.path.abspath(f) })
            if trust_type == "signedBy":
                if len(pubkeys) == 0 and len(pubkeysfile) == 0:
                    raise ValueError("At least one public key must be defined for type 'signedBy'")
            else:
                payload.append({ "type": trust_type })
            if self.args.debug:
                util.write_out(str(payload))
            policy["transports"][sstype][registry] = payload
            policy_file.seek(0)
            json.dump(policy, policy_file, indent=4)
            policy_file.truncate()
            if sigstore:
                if sstype == "atomic":
                    raise ValueError("Sigstore cannot be defined for sigstoretype 'atomic'")
                else:
                    self.modify_registry_config(registry, sstype, sigstore)

    def delete(self):
        """
        Remove trust policy entry and sigstore
        """
        sstype = self.get_sigstore_type_map(self.args.sigstoretype)
        with open(self.policy_filename, 'r+') as policy_file:
            policy = json.load(policy_file)
            try:
                del policy["transports"][sstype][self.args.registry]
            except KeyError:
                raise ValueError("Could not find trust policy defined for %s transport %s" %
                              (self.args.sigstoretype, self.args.registry))
            policy_file.seek(0)
            json.dump(policy, policy_file, indent=4)
            policy_file.truncate()
        if self.args.save:
            self.modify_registry_config(self.args.registry, self.get_sigstore_type_map(self.args.sigstoretype))

    def modify_default(self):
        """
        Modify global trust policy default
        """
        mode = "r+" if os.path.exists(self.policy_filename) else "w+"
        with open(self.policy_filename, mode) as policy_file:
            if mode == "r+":
                policy = json.load(policy_file)
            else:
                policy = {}

            default_type_map = { "accept": "insecureAcceptAnything", "reject": "reject" }
            if "default" in policy:
                policy["default"][0]["type"] = default_type_map[self.args.default_policy]
            else:
                policy["default"] = [ { "type": default_type_map[self.args.default_policy] }]

            policy_file.seek(0)
            json.dump(policy, policy_file, indent=4)
            policy_file.truncate()

    def get_pubkey_data(self, key_reference):
        """
        Return public key base64 encoded string to embed as keyData in policy.json
        :param key_reference: local file or download URI of public key
                              also accepts local user GPG keyring ID
        :return: encoded pubkey string or False
        """
        try:
            from urlparse import urlparse #pylint: disable=import-error
        except ImportError:
            from urllib.parse import urlparse #pylint: disable=no-name-in-module,import-error

        keydata = None
        token = urlparse(key_reference)
        if not token.scheme or not token.netloc:
            if not os.path.exists(key_reference):
                cmd = ["gpg2", "--armor", "--export", key_reference]
                keydata = util.check_output(cmd)
                if not keydata:
                    raise ValueError("The public key reference '%s' was not "
                                     "found as a file or in the user's GPG "
                                     "keyring. Do you have any keys listed "
                                     "with 'gpg2 --list-keys'? Or try "
                                     "setting 'GNUPGHOME=~/.gnupg' or edit "
                                     "'/etc/atomic.conf' to reference user's "
                                     "GPG keyring." % key_reference)
            else:
                with open(key_reference, 'r') as f:
                    keydata = f.read()
        else:
            if token.scheme != "https":
                if not self.args.assumeyes:
                    confirm = util.input("Are you sure you want to download this public key using insecure '%s' transport? (y/N) " % token.scheme)
                    if not "y" in confirm.lower():
                        raise ValueError("Aborting 'trust add' due to insecure download of public key from %s." % key_reference)
            if self.args.debug:
                util.write_out("Downloading key from %s" % key_reference)
            r = requests.get(key_reference)
            if r.status_code == 200:
                keydata = r.content
            else:
                raise ValueError("Could not download public key from %s. Status code %s." % (key_reference, r.status_code))
        return b64encode(keydata)

    def check_policy(self, policy, sstype):
        """
        Prep the policy file with required data structure
        :param policy: policy data
        :param sstype: sigstore type
        :return: modified policy
        """
        if not "transports" in policy:
            policy["transports"] = {}
        if not sstype in policy["transports"]:
            policy["transports"][sstype] = {}
        return policy

    def modify_registry_config(self, registry, sstype, sigstore=None):
        """
        Modify the registries.d configuration for a registry
        :param registry: a registry name
        :param sstype: mapped sigstore type
        :param sigstore: a file:/// or https:// URL. Optional
        """
        registry_config_path = util.get_atomic_config_item(["registry_confdir"], self.atomic_config)
        registry_configs, _ = util.get_registry_configs(registry_config_path)
        reg_info = util.have_match_registry(registry, registry_configs)
        reg_yaml_file = "%s.yaml" % os.path.join(registry_config_path, registry.replace("/", "-"))
        mode = "r+"
        if (not reg_info or registry not in registry_configs):
            if not os.path.exists(reg_yaml_file):
                mode="w+"
        else:
            if not sigstore == registry_configs[registry]["sigstore"]:
                reg_yaml_file = registry_configs[registry]["filename"]
        self.write_registry_config_file(reg_yaml_file, registry, sstype, sigstore, mode=mode)

    def write_registry_config_file(self, reg_file, registry, sstype="docker", sigstore=None, mode="r+"):
        """
        Utility method to modify existing registry config file
        :param reg_file: registry filename
        :param registry: registry name
        :param sstype: mapped sigstore type
        :param sigstore: sigstore server. If None the sigstore section is deleted.
        :param mode: file open mode
        """
        with open(reg_file, mode) as f:
            d = yaml.load(f)
            if not sigstore:
                if d:
                    del d[sstype][registry]
            else:
                d = { sstype: {}}
                d[sstype][str(registry)] = { "sigstore": str(sigstore) }
            f.seek(0)
            yaml.dump(d, f, default_flow_style=False)
            f.truncate()

    def get_sigstore_type_map(self, sigstore_type):
        """
        Given the user-friendly trust type, get the skopeo value
        :param sigstore_type: one of web,local,atomic
        :return: skopeo trust policy type string
        """
        t = { "docker": "docker", "web": "docker", "local": "dir", "dir": "dir", "atomic": "atomic" }
        if sigstore_type not in t:
            raise ValueError("Invalid sigstore type %s" % sigstore_type)
        return t[sigstore_type]

    def discover_sigstore(self, pull_image):
        """
        Check for registry/repo/sigstore metadata image
        prompt user for trust on first use workflow
        :param pull_image: image being pulled, used to discover matching sigstore meta image
        """
        if not util.get_atomic_config_item(['discover_sigstores'], util.get_atomic_config()):
            return True
        registry, repo, _, _, _ = util.Decompose(pull_image).all
        repo = repo.split('/')
        registry_config_path = util.get_atomic_config_item(["registry_confdir"], self.atomic_config)
        registry_configs, _ = util.get_registry_configs(registry_config_path)
        sigstore_labels = False
        scope = '/'.join([registry, repo[0]])
        if not scope in registry_configs:
            sigstore_labels = self.get_sigstore_image_metadata(registry, repo[0])
            if not sigstore_labels:
                scope = registry
                if not scope in registry_configs:
                    sigstore_labels = self.get_sigstore_image_metadata(registry)
        if self._validate_sigstore_labels(sigstore_labels):
            if self.prompt_trust(sigstore_labels):
                explicit_sigstoretype = "web"
                if "sigstore-type" in sigstore_labels:
                    explicit_sigstoretype = sigstore_labels['sigstore-type']
                self.add(registry=scope, trust_type="signedBy", sigstoretype=explicit_sigstoretype, keytype="GPGKeys", pubkeys=[sigstore_labels['pubkey-url']], sigstore=sigstore_labels['sigstore-url'])

    def get_sigstore_image_metadata(self, registry, repo=None):
        """
        Get sigstore metadata image
        :param registry: registry string
        :param repo: repo string
        :return dict of labels or False
        """
        _img = util.get_atomic_config_item(['sigstore_metadata_image'], util.get_atomic_config())
        if repo:
            sigstoreimage = '/'.join([registry, repo, _img])
        else:
            sigstoreimage = '/'.join([registry, _img])
        try:
            data = util.skopeo_inspect("docker://" + sigstoreimage, args=None)
        except ValueError:
            data = None
        if data:
            return data['Labels']
        else:
            return False

    def _validate_sigstore_labels(self, labels):
        """
        Validate sigstore metadata.
        If there's a missing key or something we don't want to perform any automatic trust policy configuration
        :param labels: unvalidated labels. Should be either dict or False
        :return: True if labels are valid
        """
        is_valid = False
        if labels:
            # sigstore-type is optional
            expected_keys = ["pubkey-id", "pubkey-fingerprint", "pubkey-url", "sigstore-url"]
            for k in expected_keys:
                is_valid = k in labels
        return is_valid

    def prompt_trust(self, labels):
        """
        Prompt user for trust on first use workflow
        :param labels: dict of metadata labels defining sigstore trust
        :return: True if user accepts
        """
        util.write_out("ID: " + labels['pubkey-id'])
        util.write_out("Fingerprint: " + labels['pubkey-fingerprint'])
        util.write_out("Public key download URL: %s" % labels['pubkey-url'])
        if not self.args.assumeyes:
            confirm = util.input("Do you want to add trust policy for this registry? (y/N) ")
            if not "y" in confirm.lower():
                return False
        return True

    def _get_policy(self):
        policy = self.default_policy_file
        mode = "r+" if os.path.exists(self.policy_filename) else "w+"
        with open(self.policy_filename, mode) as policy_file:
            if mode == "r+":
                policy = json.load(policy_file)
            else:
                policy_file.seek(0)
                json.dump(policy, policy_file, indent=4)
                policy_file.truncate()

        return policy

    @property
    def default_policy_file(self):
        '''
        Return default policy file
        '''
        return { "default": [{ "type": "insecureAcceptAnything" }], "transports": { "docker-daemon": { "": [{ "type": "insecureAcceptAnything" }]}}}

    def show_json(self, policy=None):
        if not policy:
            policy=self._get_policy()
        table = {}
        registry_config_path = util.get_atomic_config_item(["registry_confdir"], self.atomic_config)
        registry_configs, _ = util.get_registry_configs(registry_config_path)
        if "transports" in policy:
            for tt in TRANSPORT_TYPES:
                if tt in policy["transports"]:
                    for key, values in policy["transports"][tt].items():
                        table[key] = { "type": values[0]["type"] }
                        table[key]["keys"] = []
                        for v in values:
                            if "keyPath" in v:
                                table[key]["keys"].append(v["keyPath"])
                            if "keyData" in v:
                                table[key]["keys"].append(v["keyData"])
                        reg_info = util.have_match_registry(key, registry_configs)
                        if reg_info:
                            table[key]["sigstore"] = reg_info["sigstore"]
                        else:
                            table[key]["sigstore"] = ""
        if "default" in policy:
            table["* (default)"] = { "type": policy["default"][0]["type"], "keys": None, "sigstore": "" }
        return collections.OrderedDict(sorted(table.items()))

    def show(self):
        """
        Display trust policy
        If policy.json doesn't exist, create it, then display
        """
        policy=self._get_policy()
        if self.args.raw:
            util.output_json(policy)
        else:
            sorted_table = self.show_json(policy)
            if self.args.json:
                util.output_json(sorted_table)
            else:
                for key, value in sorted_table.items():
                    util.write_out('{0:<35} {1:<6} {2:<29} {3}'.format(key, self.trusttype_map(value["type"]), self.get_gpg_id(value["keys"]), value["sigstore"]))

    def reset(self):
        """
        Reset trust policy by
         overwriting policy.json
         removing all files in registries.d except default.yaml
        """
        if not self.args.assumeyes:
            confirm = util.input("Are you sure you want to destroy "
                                 "trust policy for this system? "
                                 "This cannot be undone! (y/N) ")
            if not "y" in confirm.lower():
                return

        with open(self.policy_filename, "w") as policy_file:
            policy_file.seek(0)
            json.dump(self.default_policy_file, policy_file, indent=4)
            policy_file.truncate()

        registry_config_path = util.get_atomic_config_item(["registry_confdir"], self.atomic_config)
        for registries_file in os.listdir(registry_config_path):
            if registries_file != "default.yaml":
                os.remove('/'.join([registry_config_path, registries_file]))

    def get_gpg_id(self, keys):
        """
        Return GPG identity, either bracketed <email> or ID string
        comma separated if more than one key
        see gpg2 parsing documentation:
            http://git.gnupg.org/cgi-bin/gitweb.cgi?p=gnupg.git;a=blob_plain;f=doc/DETAILS
        :param keys: list of gpg key file paths (keyPath) and/or inline key payload (keyData)
        :return: comma-separated string of key ids or empty string
        """
        if not keys:
            return ""
        (keylist, tmpkey) = None, None
        for key in keys:
            if not os.path.exists(key):
                with tempfile.NamedTemporaryFile(delete=False, dir="/run/") as tmpkey:
                    tmpkey.write(b64decode(key))
                key = tmpkey.name
            cmd = ["gpg2", "--with-colons", key]
            try:
                results = util.check_output(cmd).decode('utf-8')
            except util.FileNotFound:
                results = ""
            if tmpkey:
                if os.path.exists(tmpkey.name):
                    os.remove(tmpkey.name)
            lines = results.split('\n')
            for line in lines:
                if line.startswith("uid:") or line.startswith("pub:"):
                    uid = line.split(':')[9]
                    if not uid:   #  Newer gpg2 versions output dedicated 'uid'
                        continue  #  lines and leave it blank on 'pub' lines.
                    # bracketed email
                    parsed_uid = uid.partition('<')[-1].rpartition('>')[0]
                    if not parsed_uid:
                        parsed_uid = uid
                    if keylist:
                        keylist = ",".join([keylist, parsed_uid])
                    else:
                        keylist = parsed_uid
        return keylist

    def trusttype_map(self, trust_type):
        t = { "insecureAcceptAnything": "accept", "signedBy": "signed", "reject": "reject" }
        if trust_type not in t:
            raise ValueError("Invalid trust type %s" % trust_type)
        return t[trust_type]



from Atomic import Atomic
from Atomic import util
from Atomic import info
from Atomic import update
from Atomic import verify
from Atomic import help as Help
from Atomic.mount import Mount
from Atomic.delete import Delete
import os
import math
import shutil
import tempfile
import argparse
from Atomic import backendutils

ATOMIC_CONFIG = util.get_atomic_config()
storage = ATOMIC_CONFIG.get('default_storage', "docker")

def convert_size(size):
    if size > 0:
        size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
        i = int(math.floor(math.log(size, 1000)))
        p = math.pow(1000, i)
        s = round(size/p, 2) # pylint: disable=round-builtin,old-division
        if s > 0:
            return '%s %s' % (s, size_name[i])
    return '0B'

def cli(subparser):
    # atomic images
    imagesp = subparser.add_parser("images",
                                   help=_("operate on images"))
    images_subparser = imagesp.add_subparsers(title='images subcommands',
                                              description="operate on images",
                                              help='additional help')

    # atomic images delete
    delete_parser = images_subparser.add_parser("delete",
                                                help=_("mark image for deletion"),
                                                epilog="Marks image. registry garbage-collection "
                                                "when invoked will recover used disk space")

    deletegroup = delete_parser.add_mutually_exclusive_group()
    deletegroup.add_argument("-f", "--force", default=False, dest="force",
                             action="store_true",
                             help=_("Force removal of local images, even if containers based on it exist.  Default is False"))

    deletegroup.add_argument("--remote", default=False, dest="remote",
                             action="store_true",
                             help=_("Delete image from remote repository"))

    delete_parser.add_argument("--storage", default=None, dest="storage",
                               help=_("Specify the storage from which to delete the image from. "
                                      "If not specified and there are images with the same name in "
                                      "different storages, you will be prompted to specify."))
    delete_parser.add_argument("-a", "--all", action='store_true',dest="all",
                               default=False, help=_("Delete all images"))
    delete_parser.add_argument("delete_targets", nargs='*',
                                    help=_("container image(s)"))
    delete_parser.set_defaults(_class=Delete, func='delete_image')

    if util.gomtree_available():
        generate_parser = images_subparser.add_parser("generate",
                                                  help=_("generate image 'manifests' if missing"))
        generate_parser.set_defaults(_class=Images, func='generate_validation_manifest')

        # Suppress storage flag: only docker images are supported for generate
        generate_parser.add_argument("--storage", default=storage, dest="storage",
                                     help=argparse.SUPPRESS)

    Help.cli(images_subparser)

    info.cli(images_subparser)

    # atomic images list
    list_parser = images_subparser.add_parser("list",
                                              help=_("list container images on your system"),
                                              epilog="atomic images by default will list all installed "
                                                     "container images on your system.")
    list_parser.set_defaults(_class=Images, func='display_all_image_info')

    list_parser.add_argument("-a", "--all", dest="all", default=False,
                             action="store_true",
                             help=_("Show all images, including intermediate images"))

    list_parser.add_argument("-f", "--filter", dest="filter", metavar="FILTER",
                             action="append",
                             help=_("Filter output based on VARIABLE=VALUE format"))

    list_parser.add_argument("-n", "--noheading", dest="heading", default=True,
                             action="store_false",
                             help=_("do not print heading when listing the images"))

    list_parser.add_argument("--no-trunc", dest="truncate", default=True,
                             action="store_false",
                             help=_("Don't truncate output"))

    list_parser.add_argument("-q", "--quiet", dest="quiet", default=False,
                             action="store_true",
                             help=_("Only display image IDs"))
    list_parser.add_argument("--json", action='store_true',dest="json", default=False,
                             help=_("print in a machine parseable form"))

    prune_parser = images_subparser.add_parser("prune",
                                               help=_("delete unused 'dangling' images"),
                                               epilog="Using the prune command, "
                                                      "will free up disk space deleting unused "
                                                      "'dangling' images")
    prune_parser.set_defaults(_class=Delete, func='prune_images')

    update.cli(images_subparser)

    verify.cli(images_subparser)

    info.cli_version(images_subparser)

class Images(Atomic):
    def __init__(self):
        super(Images, self).__init__()
        self.be_utils = backendutils.BackendUtils()

    def display_all_image_info(self):
        def get_col_lengths(_images):
            '''
            Determine the max length of the repository and tag names
            :param _images:
            :return: a set with len of repository and tag
            If there are no images, return 1, 1
            '''
            repo_tags = [y for x in _images if x.repotags for y in x.split_repotags]

            if repo_tags:
                return max([len(x[0]) for x in repo_tags]) + 2,\
                       max([len(x[1]) for x in repo_tags]) + 2
            else:
                return 1, 1

        if self.args.debug:
            util.write_out(str(self.args))
            self.be_utils.dump_backends()

        _images = self._get_images()
        for i in _images:
            i.repo, i.tag = i.split_repotags[0]

        if self.args.filter:
            _images = [x for x in _images if self._filter_include_image(x)]

        if self.args.json:
            util.output_json(self.return_json(_images))
            return 0

        if len(_images) == 0:
            return

        _max_repo, _max_tag = get_col_lengths(_images)

        if self.args.truncate:
            _max_id = 14
        else:
            _max_id = 65
        col_out = "{0:2} {1:" + str(_max_repo) + "} {2:" + str(_max_tag) + \
                  "} {3:" + str(_max_id) + "} {4:18} {5:14} {6:10}"

        if self.args.heading and not self.args.quiet:
            util.write_out(col_out.format(" ",
                                          "REPOSITORY",
                                          "TAG",
                                          "IMAGE ID",
                                          "CREATED",
                                          "VIRTUAL SIZE",
                                          "TYPE"))
        for image in _images:
            if self.args.quiet:
                util.write_out(image.id)

            else:
                indicator = ""
                if image.is_dangling:
                    indicator += "*"
                elif image.used:
                    indicator += ">"
                if image.vulnerable:
                    space = " " if len(indicator) < 1 else ""
                    if util.is_python2:
                        indicator = indicator + self.skull + space
                    else:
                        indicator = indicator + str(self.skull, "utf-8") + space
                _id = image.short_id if self.args.truncate else image.id
                util.write_out(col_out.format(indicator, image.repo or "<none>", image.tag or "<none>", _id, image.created[0:16],
                                              image.virtual_size, image.backend.backend))
        util.write_out("")
        return

    def _get_images(self):
        _images = self.be_utils.get_images(get_all=self.args.all)

        self._mark_used(_images)
        self._mark_vulnerable(_images)

        return _images

    def images(self):
        return self.return_json(self._get_images())


    def generate_validation_manifest(self):
        """
        Generates a gomtree validation manifest for a non-system image and stores it in
        ATOMIC_VAR_LIB
        :param:
        :return: None
        """
        _images = self.get_images(get_all=True)
        for image in _images:
            atomic_var_lib = util.ATOMIC_VAR_LIB
            if not image["RepoTags"]:
                continue
            iid = image["RepoTags"][0]
            if image["ImageType"] == "system":
                continue
            if iid == "<none>:<none>" or iid == "<none>":
                continue
            if os.path.exists(os.path.join(atomic_var_lib, "gomtree-manifests/%s.mtree" % iid)):
                continue
            manifestname = os.path.join(atomic_var_lib, "gomtree-manifests/%s.mtree" % iid)
            dname = os.path.dirname(manifestname)
            if not os.path.exists(dname):
                os.makedirs(dname)
            tmpdir = tempfile.mkdtemp()
            m = Mount()
            m.args = []
            m.image = iid
            m.mountpoint = tmpdir
            m.storage = self.args.storage
            m.mount()
            r = util.generate_validation_manifest(img_rootfs=tmpdir, keywords="type,uid,gid,mode,size,sha256digest")
            m.unmount()
            with open(manifestname,"wb",0) as f:
                f.write(r.stdout)
            shutil.rmtree(tmpdir)

    def _filter_include_image(self, image_obj):
        filterables = ["repo", "tag", "id", "image", "created", "size", "type", "is_dangling"]
        for i in self.args.filter:
            try:
                var, value = str(i).split("=")
            except ValueError:
                raise ValueError("The filter {} is not formatted correctly.  It should be VAR=VALUE".format(i))
            var = var.lower()
            if var == "repository":
                var = "repo"

            if var == "image":
                var = "id"

            if var not in filterables:
                raise ValueError("The filter {} is not valid.  "
                                 "Please choose from {}".format(var, filterables))
            if var == "type":
                var = "str_backend"

            if hasattr(image_obj, var) and value.lower() in str(getattr(image_obj, var)).lower():
                return True

        return False

    def _mark_used(self, images):
        assert isinstance(images, list)
        all_containers = [x.id for x in self.be_utils.get_containers()]
        for image in images:
            if image.id in all_containers:
                image.used = True

    def _mark_vulnerable(self, images):
        assert isinstance(images, list)
        vulnerable_uuids = self.get_vulnerable_ids()
        for image in images:
            if image.id in vulnerable_uuids:
                image.vulnerable = True

    def return_json(self, images):
        all_image_info = []
        all_vuln_info = self.get_all_vulnerable_info()
        keys = ['is_dangling', 'used', 'vulnerable', 'id', 'type', 'created', 'virtual_size']
        for img_obj in images:
            if not img_obj.repotags:
                continue
            img_dict = dict()
            img_dict['repo'], img_dict['tag'] = img_obj.repo, img_obj.tag
            for key in keys:
                img_dict[key] = getattr(img_obj, key, None)
            img_dict['vuln_info'] = \
                dict() if not img_obj.vulnerable else all_vuln_info.get(img_obj.id, None) # pylint: disable=no-member
            all_image_info.append(img_dict)
        return all_image_info





import unittest

from Atomic import mount


class TestAtomicMount(unittest.TestCase):
    def test_mount_excepts_unknown_backend(self):
        def mock_info():
            return {'Driver': 'foobardriver'}
        with mount.DockerMount('foobar') as m:
            m._info = mock_info  # pylint: disable=protected-access
            exp = 'Atomic mount is not supported on the foobardriver docker ' \
                  'storage backend.'

            # assertRaisesRegexp was deprecated by assertRaisesRegex.
            # If it is present, prefer assertRaisesRegex.
            if hasattr(self, 'assertRaisesRegex'):
                assertRaisesRegex = getattr(self, "assertRaisesRegex")
            else:
                assertRaisesRegex = getattr(self, "assertRaisesRegexp")
            assertRaisesRegex(mount.MountError, exp, m.mount, 'fedora:22')
            assertRaisesRegex(mount.MountError, exp, m.unmount)

    def test_default_options(self):
        with mount.DockerMount('foobar') as m:
            o = m.default_options([], default_con='foobar_context',
                                  default_opt=['foo', 'bar'])
            self.assertEqual(o, ['foo', 'bar', 'context="foobar_context"'])

    def test_default_options_override_defaults(self):
        with mount.DockerMount('foobar') as m:
            o = m.default_options(['override', 'opts'],
                                  default_con='foobar_context',
                                  default_opt=['will not appear'])
            self.assertEqual(o, ['override', 'opts', 'context="foobar_context"'])

    def test_default_options_no_surplus_context(self):
        with mount.DockerMount('foobar') as m:
            o = m.default_options(['ro', 'context="foobang_context"'],
                                  default_con='foobar_context')
            self.assertEqual(o, ['ro', 'context="foobang_context"'])

if __name__ == '__main__':
    unittest.main()



import os
import copy
import sys
from . import util
from . import Atomic
from .client import AtomicDocker
from docker.errors import APIError
from Atomic.backendutils import BackendUtils
from .syscontainers import OSTREE_PRESENT

try:
    from subprocess import DEVNULL  # pylint: disable=no-name-in-module
except ImportError:
    DEVNULL = open(os.devnull, 'wb')

ATOMIC_CONFIG = util.get_atomic_config()
storage = ATOMIC_CONFIG.get('default_storage', "docker")

def cli(subparser):
    # atomic containers
    c = subparser.add_parser("containers",
                             help=_("operate on containers"))
    containers_subparser = c.add_subparsers(title='containers subcommands',
                                            description="operate on containers",
                                            help='additional help')
    # atomic containers delete
    delete_parser = containers_subparser.add_parser("delete",
                                                help=_("delete specified containers"))
    delete_parser.add_argument("-f", "--force", action='store_true',
                               dest="force",
                               default=False,
                               help=_("Force removal of specified running containers"))
    delete_parser.add_argument("-a", "--all", action='store_true',dest="all",
                               default=False,
                               help=_("Delete all containers"))
    delete_parser.add_argument("--storage", default=None, dest="storage",
                               help=_("Specify the storage from which to delete the container from. "
                                      "If not specified and there are containers with the same name in "
                                      "different storages, you will be prompted to specify."))
    delete_parser.add_argument("containers", nargs='*',
                              help=_("Specify one or more containers. Must be final arguments."))
    delete_parser.set_defaults(_class=Containers, func='delete')


    # atomic containers list
    pss = containers_subparser.add_parser("list",
                                          help=_("list the containers"),
                                          epilog="By default this shows only the running containers.")
    pss.set_defaults(_class=Containers, func='ps_tty')
    pss.add_argument("-a", "--all", action='store_true',dest="all", default=False,
                     help=_("show all containers"))
    pss.add_argument("-f", "--filter", metavar='FILTER', action='append', dest="filters",
                     help=_("Filter output based on conditions given in the VARIABLE=VALUE form"))
    pss.add_argument("--json", action='store_true',dest="json", default=False,
                     help=_("print in a machine parseable form"))
    pss.add_argument("-n", "--noheading", dest="heading", default=True,
                     action="store_false",
                     help=_("do not print heading when listing the containers"))
    pss.add_argument("--no-trunc", action='store_false', dest="truncate", default=True,
                     help=_("Don't truncate output"))
    pss.add_argument("-q", "--quiet", action='store_true', dest="quiet", default=False,
                     help=_("Only display container IDs"))
    # atomic containers trim
    trimp = containers_subparser.add_parser("trim",
                                            help=_("discard unused blocks (fstrim) on running containers"),
                                            epilog="Discard unused blocks (fstrim) on rootfs of running containers.")
    trimp.set_defaults(_class=Containers, func='fstrim')

    # atomic containers update/rollback
    updatep = containers_subparser.add_parser("update",
                                              help=_("update a container"),
                                              epilog="Update the container to use a newer image.")
    updatep.set_defaults(_class=Containers, func='update')
    updatep.add_argument("container",
                         help=_("Specify one or more containers. Must be final arguments."))
    updatep.add_argument("--rebase", dest="rebase", default=None,
                         help=_("Rebase to a different image (useful for upgrading to a different tag)"))
    if OSTREE_PRESENT:
        updatep.add_argument("--set", dest="setvalues",
                             action='append',
                             help=_("Specify a variable in the VARIABLE=VALUE "
                                    "form for a system container"))

        rollbackp = containers_subparser.add_parser("rollback",
                                                    help=_("rollback a system container"),
                                                    epilog="Perform a rollback on a system container to a previous deployment.")
        rollbackp.add_argument("container", help=_("Specify the system container to rollback"))
        rollbackp.set_defaults(_class=Containers, func='rollback')

class Containers(Atomic):

    FILTER_KEYWORDS= {"container": "id", "image": "image_name", "command": "command",
                      "created": "created", "state": "state", "runtime": "runtime", "backend" : "backend.backend"}

    def __init__(self):
        super(Containers, self).__init__()
        self.beu = BackendUtils()

    def fstrim(self):
        with AtomicDocker() as client:
            for container in client.containers():
                containerId = container["Id"]
                ret = self._inspect_container(name=containerId)
                pid = ret["State"]["Pid"]
                mp = "/proc/%d/root" % (pid)
                util.write_out("Trimming container id {0}".format(containerId[0:12]))
                util.check_call(["/usr/sbin/fstrim", "-v", mp], stdout=DEVNULL)
        return

    def filter_container_objects(self, con_objs):
        def _walk(_filter_objs, _filter, _value):
            _filtered = []
            for con_obj in _filter_objs:
                it = con_obj
                for i in _filter.split("."):
                    it = getattr(it, i, None)
                if _value.lower() in it.lower():
                    _filtered.append(con_obj)
            return _filtered

        if not self.args.filters:
            return con_objs
        filtered_objs = copy.deepcopy(con_objs)
        for f in self.args.filters:
            cfilter, value = f.split('=', 1)
            cfilter = self.FILTER_KEYWORDS[cfilter]
            filtered_objs = _walk(filtered_objs, cfilter, value)
        return filtered_objs

    def ps_tty(self):
        if self.args.debug:
            util.write_out(str(self.args))
            self.beu.dump_backends()


        container_objects = self._ps()
        if not any([x.running for x in container_objects]) and not self.args.all:
            return 0

        if self.args.quiet:
            for con_obj in container_objects:
                util.write_out(con_obj.id[:12])
            return 0
        if self.args.json:
            util.output_json(self._to_json(container_objects))
            return 0

        if len(container_objects) == 0:
            return 0

        max_container_id = 12 if self.args.truncate else max([len(x.id) for x in container_objects])
        max_image_name = 20 if self.args.truncate else max([len(x.image_name) for x in container_objects])
        max_command = 20 if self.args.truncate else max([len(x.command) for x in container_objects])
        col_out = "{0:2} {1:%s} {2:%s} {3:%s} {4:16} {5:9} {6:10} {7:10}" % (max_container_id, max_image_name, max_command)
        if self.args.heading:
            util.write_out(col_out.format(" ",
                                          "CONTAINER ID",
                                          "IMAGE",
                                          "COMMAND",
                                          "CREATED",
                                          "STATE",
                                          "BACKEND",
                                          "RUNTIME"))
        for con_obj in container_objects:
            indicator = ""
            if con_obj.vulnerable:
                if util.is_python2:
                    indicator = indicator + self.skull + " "
                else:
                    indicator = indicator + str(self.skull, "utf-8") + " "
            util.write_out(col_out.format(indicator,
                                          con_obj.id[0:max_container_id],
                                          con_obj.image_name[0:max_image_name],
                                          con_obj.command[0:max_command],
                                          con_obj.created[0:16],
                                          con_obj.state[0:9],
                                          con_obj.backend.backend[0:10],
                                          con_obj.runtime[0:10]))

    def ps(self):
        container_objects = self._ps()
        return self._to_json(container_objects)

    def _ps(self):
        def _check_filters():
            if not self.args.filters:
                return True
            for f in self.args.filters:
                _filter, _ = f.split('=', 1)
                keywords = list(self.FILTER_KEYWORDS.keys())
                if _filter not in keywords:
                    raise ValueError("The filter {} is not valid.  "
                                     "Please choose from {}".format(_filter, keywords))
        _check_filters()
        containers = self.filter_container_objects(self.beu.get_containers())
        self._mark_vulnerable(containers)
        if self.args.all:
            return containers
        return [x for x in containers if x.running]

    @staticmethod
    def _to_json(con_objects):
        containers = []
        for con_obj in con_objects:
            _con = {'id': con_obj.id,
                    'image_id': con_obj.image_id,
                    'image_name': con_obj.image_name,
                    'command': con_obj.command,
                    'created': con_obj.created,
                    'state': con_obj.state,
                    'backend': con_obj.backend.backend,
                    'runtime': con_obj.runtime,
                    'vulnerable': con_obj.vulnerable,
                    'running': con_obj.running
                    }
            containers.append(_con)
        return containers

    def delete(self):
        if self.args.debug:
            util.write_out(str(self.args))
            self.beu.dump_backends()

        if (len(self.args.containers) > 0 and self.args.all) or (len(self.args.containers) < 1 and not self.args.all):
            raise ValueError("You must select --all or provide a list of images to delete.")

        if self.args.all:
            if self.args.storage:
                be = self.beu.get_backend_from_string(self.args.storage)
                container_objects = be.get_containers()
            else:
                container_objects = self.beu.get_containers()
        else:
            container_objects = []
            for con in self.args.containers:
                _, con_obj = self.beu.get_backend_and_container_obj(con, str_preferred_backend=self.args.storage or storage, required=True if self.args.storage else False)
                container_objects.append(con_obj)

        if len(container_objects) == 0:
            raise ValueError("No containers to delete")
        four_col = "   {0:12} {1:20} {2:25} {3:10}"
        if not self.args.assumeyes:
            util.write_out("Do you wish to delete the following images?\n")
        else:
            util.write_out("The following containers will be deleted.\n")
        util.write_out(four_col.format("ID", "NAME", 'IMAGE_NAME', "STORAGE"))
        for con in container_objects:
            util.write_out(four_col.format(con.id[0:12], con.name[0:20], con.image_name[0:25], con.backend.backend))
        if not self.args.assumeyes:
            confirm = util.input("\nConfirm (y/N) ")
            confirm = confirm.strip().lower()
            if not confirm in ['y', 'yes']:
                util.write_err("User aborted delete operation for {}".format(self.args.containers or "all containers"))
                sys.exit(2)

        for del_con in container_objects:
            try:
                del_con.backend.delete_container(del_con.id, force=self.args.force)
            except APIError as e:
                util.write_err("Failed to delete container {}: {}".format(con.id, e))
        return 0

    def _mark_vulnerable(self, containers):
        assert isinstance(containers, list)
        vulnerable_uuids = self.get_vulnerable_ids()
        for con in containers:
            if con.id in vulnerable_uuids:
                con.vulnerable = True

    def update(self):
        if self.syscontainers.get_checkout(self.args.container):
            return self.syscontainers.update_container(self.args.container, self.args.setvalues, self.args.rebase)
        raise ValueError("System container '%s' is not installed" % self.args.container)

    def rollback(self):
        util.write_out("Attempting to roll back system container: %s" % self.args.container)
        self.syscontainers.rollback(self.args.container)
